# [doc = "core global object\n\nThe core global object.  This is a special singleton object.  It\nis used for internal Wayland protocol features.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_display { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_display" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "fatal error event\n\nThe error event is sent out when a fatal (non-recoverable)\nerror has occurred.  The object_id argument is the object\nwhere the error occurred, most often in response to a request\nto that object.  The code identifies the error and is defined\nby the object interface.  As such, each interface defines its\nown set of error codes.  The message is a brief description\nof the error, for (debugging) convenience.\n"] fn error (& mut self , sender_id : waybackend :: types :: ObjectId , object_id : waybackend :: types :: ObjectId , code : u32 , message : & str) ; # [doc = "acknowledge object ID deletion\n\nThis event is used internally by the object ID management\nlogic. When a client deletes an object that it had created,\nthe server will send this event to acknowledge that it has\nseen the delete request. When the client receives this event,\nit will know that it can safely reuse the object ID.\n"] fn delete_id (& mut self , sender_id : waybackend :: types :: ObjectId , id : u32) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let object_id = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; let code = wire_msg . next_u32 () ; let message = wire_msg . next_string () ? ; state . error (wire_msg . sender_id () , object_id , code , message) ; Ok (()) } , 1u16 => { let id = wire_msg . next_u32 () ; state . delete_id (wire_msg . sender_id () , id) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_display" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const SYNC : u16 = 0u16 ; # [doc = "asynchronous roundtrip\n\nThe sync request asks the server to emit the 'done' event\non the returned wl_callback object.  Since requests are\nhandled in-order and events are delivered in-order, this can\nbe used as a barrier to ensure all previous requests and the\nresulting events have been handled.\n\nThe object returned by this request will be destroyed by the\ncompositor after the callback is fired and as such the client must not\nattempt to use it after that point.\n\nThe callback_data passed in the callback is undefined and should be ignored.\n"] pub fn sync (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , callback : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SYNC) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , callback) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_REGISTRY : u16 = 1u16 ; # [doc = "get global registry object\n\nThis request creates a registry object that allows the client\nto list and bind the global objects available from the\ncompositor.\n\nIt should be noted that the server side resources consumed in\nresponse to a get_registry request can only be released when the\nclient disconnects, not when the client side proxy is destroyed.\nTherefore, clients should invoke get_registry as infrequently as\npossible to avoid wasting memory.\n"] pub fn get_registry (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , registry : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_REGISTRY) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , registry) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "global error values\n\nThese errors are global and can be emitted in response to any\nserver request.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "server couldn't find object"] invalid_object = 0u32 , # [doc = "method doesn't exist on the specified interface or malformed request"] invalid_method = 1u32 , # [doc = "server is out of memory"] no_memory = 2u32 , # [doc = "implementation error in compositor"] implementation = 3u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_object : u32 = Error :: invalid_object as u32 ; const invalid_method : u32 = Error :: invalid_method as u32 ; const no_memory : u32 = Error :: no_memory as u32 ; const implementation : u32 = Error :: implementation as u32 ; match value { invalid_object => Ok (Self :: invalid_object) , invalid_method => Ok (Self :: invalid_method) , no_memory => Ok (Self :: no_memory) , implementation => Ok (Self :: implementation) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "global registry object\n\nThe singleton global registry object.  The server has a number of\nglobal objects that are available to all clients.  These objects\ntypically represent an actual object in the server (for example,\nan input device) or they are singleton objects that provide\nextension functionality.\n\nWhen a client creates a registry object, the registry object\nwill emit a global event for each global currently in the\nregistry.  Globals come and go as a result of device or\nmonitor hotplugs, reconfiguration or other events, and the\nregistry will send out global and global_remove events to\nkeep the client up to date with the changes.  To mark the end\nof the initial burst of events, the client can use the\nwl_display.sync request immediately after calling\nwl_display.get_registry.\n\nA client can bind to a global object by using the bind\nrequest.  This creates a client-side handle that lets the object\nemit events to the client and lets the client invoke requests on\nthe object.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_registry { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_registry" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "announce global object\n\nNotify the client of global objects.\n\nThe event notifies the client that a global object with\nthe given name is now available, and it implements the\ngiven version of the given interface.\n"] fn global (& mut self , sender_id : waybackend :: types :: ObjectId , name : u32 , interface : & str , version : u32) ; # [doc = "announce removal of global object\n\nNotify the client of removed global objects.\n\nThis event notifies the client that the global identified\nby name is no longer available.  If the client bound to\nthe global using the bind request, the client should now\ndestroy that object.\n\nThe object remains valid and requests to the object will be\nignored until the client destroys it, to avoid races between\nthe global going away and a client sending a request to it.\n"] fn global_remove (& mut self , sender_id : waybackend :: types :: ObjectId , name : u32) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let name = wire_msg . next_u32 () ; let interface = wire_msg . next_string () ? ; let version = wire_msg . next_u32 () ; state . global (wire_msg . sender_id () , name , interface , version) ; Ok (()) } , 1u16 => { let name = wire_msg . next_u32 () ; state . global_remove (wire_msg . sender_id () , name) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_registry" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const BIND : u16 = 0u16 ; # [doc = "bind an object to the display\n\nBinds a new, client-created object to the server using the\nspecified name as the identifier.\n"] pub fn bind (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , name : u32 , id : waybackend :: types :: ObjectId , interface : & str , version : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , BIND) ? ; wire_msg_builder . add_u32 (wayland_fd , name) ? ; wire_msg_builder . add_new_unspecified_id (wayland_fd , id , interface , version) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "callback object\n\nClients can handle the 'done' event to get notified when\nthe related request is done.\n\nNote, because wl_callback objects are created from multiple independent\nfactory interfaces, the wl_callback interface is frozen at version 1.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_callback { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_callback" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "done event\n\nNotify the client when the related request is done.\n\nTHIS IS A DESTRUCTOR"] fn done (& mut self , sender_id : waybackend :: types :: ObjectId , callback_data : u32) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let callback_data = wire_msg . next_u32 () ; state . done (wire_msg . sender_id () , callback_data) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_callback" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; } } # [doc = "the compositor singleton\n\nA compositor.  This object is a singleton global.  The\ncompositor is in charge of combining the contents of multiple\nsurfaces into one displayable output.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_compositor { use super :: * ; pub const MAX_VERSION : u32 = 6u32 ; pub const NAME : & str = "wl_compositor" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_compositor" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const CREATE_SURFACE : u16 = 0u16 ; # [doc = "create new surface\n\nAsk the compositor to create a new surface.\n"] pub fn create_surface (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , CREATE_SURFACE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } pub const CREATE_REGION : u16 = 1u16 ; # [doc = "create new region\n\nAsk the compositor to create a new region.\n"] pub fn create_region (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , CREATE_REGION) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "a shared memory pool\n\nThe wl_shm_pool object encapsulates a piece of memory shared\nbetween the compositor and client.  Through the wl_shm_pool\nobject, the client can allocate shared memory wl_buffer objects.\nAll objects created through the same pool share the same\nunderlying mapped memory. Reusing the mapped memory avoids the\nsetup/teardown overhead and is useful when interactively resizing\na surface or for many small buffers.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_shm_pool { use super :: * ; pub const MAX_VERSION : u32 = 2u32 ; pub const NAME : & str = "wl_shm_pool" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_shm_pool" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const CREATE_BUFFER : u16 = 0u16 ; # [doc = "create a buffer from the pool\n\nCreate a wl_buffer object from the pool.\n\nThe buffer is created offset bytes into the pool and has\nwidth and height as specified.  The stride argument specifies\nthe number of bytes from the beginning of one row to the beginning\nof the next.  The format is the pixel format of the buffer and\nmust be one of those advertised through the wl_shm.format event.\n\nA buffer will keep a reference to the pool it was created from\nso it is valid to destroy the pool immediately after creating\na buffer from it.\n"] pub fn create_buffer (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , offset : i32 , width : i32 , height : i32 , stride : i32 , format : wl_shm :: Format) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , CREATE_BUFFER) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_i32 (wayland_fd , offset) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . add_i32 (wayland_fd , stride) ? ; wire_msg_builder . add_u32 (wayland_fd , format . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DESTROY : u16 = 1u16 ; # [doc = "destroy the pool\n\nDestroy the shared memory pool.\n\nThe mmapped memory will be released when all\nbuffers that have been created from this pool\nare gone.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RESIZE : u16 = 2u16 ; # [doc = "change the size of the pool mapping\n\nThis request will cause the server to remap the backing memory\nfor the pool from the file descriptor passed when the pool was\ncreated, but using the new size.  This request can only be\nused to make the pool bigger.\n\nThis request only changes the amount of bytes that are mmapped\nby the server and does not touch the file corresponding to the\nfile descriptor passed at creation time. It is the client's\nresponsibility to ensure that the file is at least as big as\nthe new pool size.\n"] pub fn resize (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , size : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RESIZE) ? ; wire_msg_builder . add_i32 (wayland_fd , size) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "shared memory support\n\nA singleton global object that provides support for shared\nmemory.\n\nClients can create wl_shm_pool objects using the create_pool\nrequest.\n\nOn binding the wl_shm object one or more format events\nare emitted to inform clients about the valid pixel formats\nthat can be used for buffers.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_shm { use super :: * ; pub const MAX_VERSION : u32 = 2u32 ; pub const NAME : & str = "wl_shm" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "pixel format description\n\nInforms the client about a valid pixel format that\ncan be used for buffers. Known formats include\nargb8888 and xrgb8888.\n"] fn format (& mut self , sender_id : waybackend :: types :: ObjectId , format : Format) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let format : Format = wire_msg . next_u32 () . try_into () ? ; state . format (wire_msg . sender_id () , format) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_shm" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const CREATE_POOL : u16 = 0u16 ; # [doc = "create a shm pool\n\nCreate a new wl_shm_pool object.\n\nThe pool can be used to create shared memory based buffer\nobjects.  The server will mmap size bytes of the passed file\ndescriptor, to use as backing memory for the pool.\n"] pub fn create_pool (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , fd : & impl waybackend :: rustix :: fd :: AsRawFd , size : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , CREATE_POOL) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_fd (wayland_fd , fd) ? ; wire_msg_builder . add_i32 (wayland_fd , size) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RELEASE : u16 = 1u16 ; # [doc = "release the shm object\n\nUsing this request a client can tell the server that it is not going to\nuse the shm object anymore.\n\nObjects created via this interface remain unaffected.\n\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "wl_shm error values\n\nThese errors can be emitted in response to wl_shm requests.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "buffer format is not known"] invalid_format = 0u32 , # [doc = "invalid size or stride during pool or buffer creation"] invalid_stride = 1u32 , # [doc = "mmapping the file descriptor failed"] invalid_fd = 2u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_format : u32 = Error :: invalid_format as u32 ; const invalid_stride : u32 = Error :: invalid_stride as u32 ; const invalid_fd : u32 = Error :: invalid_fd as u32 ; match value { invalid_format => Ok (Self :: invalid_format) , invalid_stride => Ok (Self :: invalid_stride) , invalid_fd => Ok (Self :: invalid_fd) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } # [doc = "pixel formats\n\nThis describes the memory layout of an individual pixel.\n\nAll renderers should support argb8888 and xrgb8888 but any other\nformats are optional and may not be supported by the particular\nrenderer in use.\n\nThe drm format codes match the macros defined in drm_fourcc.h, except\nargb8888 and xrgb8888. The formats actually supported by the compositor\nwill be reported by the format event.\n\nFor all wl_shm formats and unless specified in another protocol\nextension, pre-multiplied alpha is used for pixel values.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Format { # [doc = "32-bit ARGB format, [31:0] A:R:G:B 8:8:8:8 little endian"] argb8888 = 0u32 , # [doc = "32-bit RGB format, [31:0] x:R:G:B 8:8:8:8 little endian"] xrgb8888 = 1u32 , # [doc = "8-bit color index format, [7:0] C"] c8 = 538982467u32 , # [doc = "8-bit RGB format, [7:0] R:G:B 3:3:2"] rgb332 = 943867730u32 , # [doc = "8-bit BGR format, [7:0] B:G:R 2:3:3"] bgr233 = 944916290u32 , # [doc = "16-bit xRGB format, [15:0] x:R:G:B 4:4:4:4 little endian"] xrgb4444 = 842093144u32 , # [doc = "16-bit xBGR format, [15:0] x:B:G:R 4:4:4:4 little endian"] xbgr4444 = 842089048u32 , # [doc = "16-bit RGBx format, [15:0] R:G:B:x 4:4:4:4 little endian"] rgbx4444 = 842094674u32 , # [doc = "16-bit BGRx format, [15:0] B:G:R:x 4:4:4:4 little endian"] bgrx4444 = 842094658u32 , # [doc = "16-bit ARGB format, [15:0] A:R:G:B 4:4:4:4 little endian"] argb4444 = 842093121u32 , # [doc = "16-bit ABGR format, [15:0] A:B:G:R 4:4:4:4 little endian"] abgr4444 = 842089025u32 , # [doc = "16-bit RBGA format, [15:0] R:G:B:A 4:4:4:4 little endian"] rgba4444 = 842088786u32 , # [doc = "16-bit BGRA format, [15:0] B:G:R:A 4:4:4:4 little endian"] bgra4444 = 842088770u32 , # [doc = "16-bit xRGB format, [15:0] x:R:G:B 1:5:5:5 little endian"] xrgb1555 = 892424792u32 , # [doc = "16-bit xBGR 1555 format, [15:0] x:B:G:R 1:5:5:5 little endian"] xbgr1555 = 892420696u32 , # [doc = "16-bit RGBx 5551 format, [15:0] R:G:B:x 5:5:5:1 little endian"] rgbx5551 = 892426322u32 , # [doc = "16-bit BGRx 5551 format, [15:0] B:G:R:x 5:5:5:1 little endian"] bgrx5551 = 892426306u32 , # [doc = "16-bit ARGB 1555 format, [15:0] A:R:G:B 1:5:5:5 little endian"] argb1555 = 892424769u32 , # [doc = "16-bit ABGR 1555 format, [15:0] A:B:G:R 1:5:5:5 little endian"] abgr1555 = 892420673u32 , # [doc = "16-bit RGBA 5551 format, [15:0] R:G:B:A 5:5:5:1 little endian"] rgba5551 = 892420434u32 , # [doc = "16-bit BGRA 5551 format, [15:0] B:G:R:A 5:5:5:1 little endian"] bgra5551 = 892420418u32 , # [doc = "16-bit RGB 565 format, [15:0] R:G:B 5:6:5 little endian"] rgb565 = 909199186u32 , # [doc = "16-bit BGR 565 format, [15:0] B:G:R 5:6:5 little endian"] bgr565 = 909199170u32 , # [doc = "24-bit RGB format, [23:0] R:G:B little endian"] rgb888 = 875710290u32 , # [doc = "24-bit BGR format, [23:0] B:G:R little endian"] bgr888 = 875710274u32 , # [doc = "32-bit xBGR format, [31:0] x:B:G:R 8:8:8:8 little endian"] xbgr8888 = 875709016u32 , # [doc = "32-bit RGBx format, [31:0] R:G:B:x 8:8:8:8 little endian"] rgbx8888 = 875714642u32 , # [doc = "32-bit BGRx format, [31:0] B:G:R:x 8:8:8:8 little endian"] bgrx8888 = 875714626u32 , # [doc = "32-bit ABGR format, [31:0] A:B:G:R 8:8:8:8 little endian"] abgr8888 = 875708993u32 , # [doc = "32-bit RGBA format, [31:0] R:G:B:A 8:8:8:8 little endian"] rgba8888 = 875708754u32 , # [doc = "32-bit BGRA format, [31:0] B:G:R:A 8:8:8:8 little endian"] bgra8888 = 875708738u32 , # [doc = "32-bit xRGB format, [31:0] x:R:G:B 2:10:10:10 little endian"] xrgb2101010 = 808669784u32 , # [doc = "32-bit xBGR format, [31:0] x:B:G:R 2:10:10:10 little endian"] xbgr2101010 = 808665688u32 , # [doc = "32-bit RGBx format, [31:0] R:G:B:x 10:10:10:2 little endian"] rgbx1010102 = 808671314u32 , # [doc = "32-bit BGRx format, [31:0] B:G:R:x 10:10:10:2 little endian"] bgrx1010102 = 808671298u32 , # [doc = "32-bit ARGB format, [31:0] A:R:G:B 2:10:10:10 little endian"] argb2101010 = 808669761u32 , # [doc = "32-bit ABGR format, [31:0] A:B:G:R 2:10:10:10 little endian"] abgr2101010 = 808665665u32 , # [doc = "32-bit RGBA format, [31:0] R:G:B:A 10:10:10:2 little endian"] rgba1010102 = 808665426u32 , # [doc = "32-bit BGRA format, [31:0] B:G:R:A 10:10:10:2 little endian"] bgra1010102 = 808665410u32 , # [doc = "packed YCbCr format, [31:0] Cr0:Y1:Cb0:Y0 8:8:8:8 little endian"] yuyv = 1448695129u32 , # [doc = "packed YCbCr format, [31:0] Cb0:Y1:Cr0:Y0 8:8:8:8 little endian"] yvyu = 1431918169u32 , # [doc = "packed YCbCr format, [31:0] Y1:Cr0:Y0:Cb0 8:8:8:8 little endian"] uyvy = 1498831189u32 , # [doc = "packed YCbCr format, [31:0] Y1:Cb0:Y0:Cr0 8:8:8:8 little endian"] vyuy = 1498765654u32 , # [doc = "packed AYCbCr format, [31:0] A:Y:Cb:Cr 8:8:8:8 little endian"] ayuv = 1448433985u32 , # [doc = "2 plane YCbCr Cr:Cb format, 2x2 subsampled Cr:Cb plane"] nv12 = 842094158u32 , # [doc = "2 plane YCbCr Cb:Cr format, 2x2 subsampled Cb:Cr plane"] nv21 = 825382478u32 , # [doc = "2 plane YCbCr Cr:Cb format, 2x1 subsampled Cr:Cb plane"] nv16 = 909203022u32 , # [doc = "2 plane YCbCr Cb:Cr format, 2x1 subsampled Cb:Cr plane"] nv61 = 825644622u32 , # [doc = "3 plane YCbCr format, 4x4 subsampled Cb (1) and Cr (2) planes"] yuv410 = 961959257u32 , # [doc = "3 plane YCbCr format, 4x4 subsampled Cr (1) and Cb (2) planes"] yvu410 = 961893977u32 , # [doc = "3 plane YCbCr format, 4x1 subsampled Cb (1) and Cr (2) planes"] yuv411 = 825316697u32 , # [doc = "3 plane YCbCr format, 4x1 subsampled Cr (1) and Cb (2) planes"] yvu411 = 825316953u32 , # [doc = "3 plane YCbCr format, 2x2 subsampled Cb (1) and Cr (2) planes"] yuv420 = 842093913u32 , # [doc = "3 plane YCbCr format, 2x2 subsampled Cr (1) and Cb (2) planes"] yvu420 = 842094169u32 , # [doc = "3 plane YCbCr format, 2x1 subsampled Cb (1) and Cr (2) planes"] yuv422 = 909202777u32 , # [doc = "3 plane YCbCr format, 2x1 subsampled Cr (1) and Cb (2) planes"] yvu422 = 909203033u32 , # [doc = "3 plane YCbCr format, non-subsampled Cb (1) and Cr (2) planes"] yuv444 = 875713881u32 , # [doc = "3 plane YCbCr format, non-subsampled Cr (1) and Cb (2) planes"] yvu444 = 875714137u32 , # [doc = "[7:0] R"] r8 = 538982482u32 , # [doc = "[15:0] R little endian"] r16 = 540422482u32 , # [doc = "[15:0] R:G 8:8 little endian"] rg88 = 943212370u32 , # [doc = "[15:0] G:R 8:8 little endian"] gr88 = 943215175u32 , # [doc = "[31:0] R:G 16:16 little endian"] rg1616 = 842221394u32 , # [doc = "[31:0] G:R 16:16 little endian"] gr1616 = 842224199u32 , # [doc = "[63:0] x:R:G:B 16:16:16:16 little endian"] xrgb16161616f = 1211388504u32 , # [doc = "[63:0] x:B:G:R 16:16:16:16 little endian"] xbgr16161616f = 1211384408u32 , # [doc = "[63:0] A:R:G:B 16:16:16:16 little endian"] argb16161616f = 1211388481u32 , # [doc = "[63:0] A:B:G:R 16:16:16:16 little endian"] abgr16161616f = 1211384385u32 , # [doc = "[31:0] X:Y:Cb:Cr 8:8:8:8 little endian"] xyuv8888 = 1448434008u32 , # [doc = "[23:0] Cr:Cb:Y 8:8:8 little endian"] vuy888 = 875713878u32 , # [doc = "Y followed by U then V, 10:10:10. Non-linear modifier only"] vuy101010 = 808670550u32 , # [doc = "[63:0] Cr0:0:Y1:0:Cb0:0:Y0:0 10:6:10:6:10:6:10:6 little endian per 2 Y pixels"] y210 = 808530521u32 , # [doc = "[63:0] Cr0:0:Y1:0:Cb0:0:Y0:0 12:4:12:4:12:4:12:4 little endian per 2 Y pixels"] y212 = 842084953u32 , # [doc = "[63:0] Cr0:Y1:Cb0:Y0 16:16:16:16 little endian per 2 Y pixels"] y216 = 909193817u32 , # [doc = "[31:0] A:Cr:Y:Cb 2:10:10:10 little endian"] y410 = 808531033u32 , # [doc = "[63:0] A:0:Cr:0:Y:0:Cb:0 12:4:12:4:12:4:12:4 little endian"] y412 = 842085465u32 , # [doc = "[63:0] A:Cr:Y:Cb 16:16:16:16 little endian"] y416 = 909194329u32 , # [doc = "[31:0] X:Cr:Y:Cb 2:10:10:10 little endian"] xvyu2101010 = 808670808u32 , # [doc = "[63:0] X:0:Cr:0:Y:0:Cb:0 12:4:12:4:12:4:12:4 little endian"] xvyu12_16161616 = 909334104u32 , # [doc = "[63:0] X:Cr:Y:Cb 16:16:16:16 little endian"] xvyu16161616 = 942954072u32 , # [doc = "[63:0]   A3:A2:Y3:0:Cr0:0:Y2:0:A1:A0:Y1:0:Cb0:0:Y0:0  1:1:8:2:8:2:8:2:1:1:8:2:8:2:8:2 little endian"] y0l0 = 810299481u32 , # [doc = "[63:0]   X3:X2:Y3:0:Cr0:0:Y2:0:X1:X0:Y1:0:Cb0:0:Y0:0  1:1:8:2:8:2:8:2:1:1:8:2:8:2:8:2 little endian"] x0l0 = 810299480u32 , # [doc = "[63:0]   A3:A2:Y3:Cr0:Y2:A1:A0:Y1:Cb0:Y0  1:1:10:10:10:1:1:10:10:10 little endian"] y0l2 = 843853913u32 , # [doc = "[63:0]   X3:X2:Y3:Cr0:Y2:X1:X0:Y1:Cb0:Y0  1:1:10:10:10:1:1:10:10:10 little endian"] x0l2 = 843853912u32 , # [doc = ""] yuv420_8bit = 942691673u32 , # [doc = ""] yuv420_10bit = 808539481u32 , # [doc = ""] xrgb8888_a8 = 943805016u32 , # [doc = ""] xbgr8888_a8 = 943800920u32 , # [doc = ""] rgbx8888_a8 = 943806546u32 , # [doc = ""] bgrx8888_a8 = 943806530u32 , # [doc = ""] rgb888_a8 = 943798354u32 , # [doc = ""] bgr888_a8 = 943798338u32 , # [doc = ""] rgb565_a8 = 943797586u32 , # [doc = ""] bgr565_a8 = 943797570u32 , # [doc = "non-subsampled Cr:Cb plane"] nv24 = 875714126u32 , # [doc = "non-subsampled Cb:Cr plane"] nv42 = 842290766u32 , # [doc = "2x1 subsampled Cr:Cb plane, 10 bit per channel"] p210 = 808530512u32 , # [doc = "2x2 subsampled Cr:Cb plane 10 bits per channel"] p010 = 808530000u32 , # [doc = "2x2 subsampled Cr:Cb plane 12 bits per channel"] p012 = 842084432u32 , # [doc = "2x2 subsampled Cr:Cb plane 16 bits per channel"] p016 = 909193296u32 , # [doc = "[63:0] A:x:B:x:G:x:R:x 10:6:10:6:10:6:10:6 little endian"] axbxgxrx106106106106 = 808534593u32 , # [doc = "2x2 subsampled Cr:Cb plane"] nv15 = 892425806u32 , # [doc = ""] q410 = 808531025u32 , # [doc = ""] q401 = 825242705u32 , # [doc = "[63:0] x:R:G:B 16:16:16:16 little endian"] xrgb16161616 = 942953048u32 , # [doc = "[63:0] x:B:G:R 16:16:16:16 little endian"] xbgr16161616 = 942948952u32 , # [doc = "[63:0] A:R:G:B 16:16:16:16 little endian"] argb16161616 = 942953025u32 , # [doc = "[63:0] A:B:G:R 16:16:16:16 little endian"] abgr16161616 = 942948929u32 , # [doc = "[7:0] C0:C1:C2:C3:C4:C5:C6:C7 1:1:1:1:1:1:1:1 eight pixels/byte"] c1 = 538980675u32 , # [doc = "[7:0] C0:C1:C2:C3 2:2:2:2 four pixels/byte"] c2 = 538980931u32 , # [doc = "[7:0] C0:C1 4:4 two pixels/byte"] c4 = 538981443u32 , # [doc = "[7:0] D0:D1:D2:D3:D4:D5:D6:D7 1:1:1:1:1:1:1:1 eight pixels/byte"] d1 = 538980676u32 , # [doc = "[7:0] D0:D1:D2:D3 2:2:2:2 four pixels/byte"] d2 = 538980932u32 , # [doc = "[7:0] D0:D1 4:4 two pixels/byte"] d4 = 538981444u32 , # [doc = "[7:0] D"] d8 = 538982468u32 , # [doc = "[7:0] R0:R1:R2:R3:R4:R5:R6:R7 1:1:1:1:1:1:1:1 eight pixels/byte"] r1 = 538980690u32 , # [doc = "[7:0] R0:R1:R2:R3 2:2:2:2 four pixels/byte"] r2 = 538980946u32 , # [doc = "[7:0] R0:R1 4:4 two pixels/byte"] r4 = 538981458u32 , # [doc = "[15:0] x:R 6:10 little endian"] r10 = 540029266u32 , # [doc = "[15:0] x:R 4:12 little endian"] r12 = 540160338u32 , # [doc = "[31:0] A:Cr:Cb:Y 8:8:8:8 little endian"] avuy8888 = 1498764865u32 , # [doc = "[31:0] X:Cr:Cb:Y 8:8:8:8 little endian"] xvuy8888 = 1498764888u32 , # [doc = "2x2 subsampled Cr:Cb plane 10 bits per channel packed"] p030 = 808661072u32 , } impl std :: convert :: TryFrom < u32 > for Format { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const argb8888 : u32 = Format :: argb8888 as u32 ; const xrgb8888 : u32 = Format :: xrgb8888 as u32 ; const c8 : u32 = Format :: c8 as u32 ; const rgb332 : u32 = Format :: rgb332 as u32 ; const bgr233 : u32 = Format :: bgr233 as u32 ; const xrgb4444 : u32 = Format :: xrgb4444 as u32 ; const xbgr4444 : u32 = Format :: xbgr4444 as u32 ; const rgbx4444 : u32 = Format :: rgbx4444 as u32 ; const bgrx4444 : u32 = Format :: bgrx4444 as u32 ; const argb4444 : u32 = Format :: argb4444 as u32 ; const abgr4444 : u32 = Format :: abgr4444 as u32 ; const rgba4444 : u32 = Format :: rgba4444 as u32 ; const bgra4444 : u32 = Format :: bgra4444 as u32 ; const xrgb1555 : u32 = Format :: xrgb1555 as u32 ; const xbgr1555 : u32 = Format :: xbgr1555 as u32 ; const rgbx5551 : u32 = Format :: rgbx5551 as u32 ; const bgrx5551 : u32 = Format :: bgrx5551 as u32 ; const argb1555 : u32 = Format :: argb1555 as u32 ; const abgr1555 : u32 = Format :: abgr1555 as u32 ; const rgba5551 : u32 = Format :: rgba5551 as u32 ; const bgra5551 : u32 = Format :: bgra5551 as u32 ; const rgb565 : u32 = Format :: rgb565 as u32 ; const bgr565 : u32 = Format :: bgr565 as u32 ; const rgb888 : u32 = Format :: rgb888 as u32 ; const bgr888 : u32 = Format :: bgr888 as u32 ; const xbgr8888 : u32 = Format :: xbgr8888 as u32 ; const rgbx8888 : u32 = Format :: rgbx8888 as u32 ; const bgrx8888 : u32 = Format :: bgrx8888 as u32 ; const abgr8888 : u32 = Format :: abgr8888 as u32 ; const rgba8888 : u32 = Format :: rgba8888 as u32 ; const bgra8888 : u32 = Format :: bgra8888 as u32 ; const xrgb2101010 : u32 = Format :: xrgb2101010 as u32 ; const xbgr2101010 : u32 = Format :: xbgr2101010 as u32 ; const rgbx1010102 : u32 = Format :: rgbx1010102 as u32 ; const bgrx1010102 : u32 = Format :: bgrx1010102 as u32 ; const argb2101010 : u32 = Format :: argb2101010 as u32 ; const abgr2101010 : u32 = Format :: abgr2101010 as u32 ; const rgba1010102 : u32 = Format :: rgba1010102 as u32 ; const bgra1010102 : u32 = Format :: bgra1010102 as u32 ; const yuyv : u32 = Format :: yuyv as u32 ; const yvyu : u32 = Format :: yvyu as u32 ; const uyvy : u32 = Format :: uyvy as u32 ; const vyuy : u32 = Format :: vyuy as u32 ; const ayuv : u32 = Format :: ayuv as u32 ; const nv12 : u32 = Format :: nv12 as u32 ; const nv21 : u32 = Format :: nv21 as u32 ; const nv16 : u32 = Format :: nv16 as u32 ; const nv61 : u32 = Format :: nv61 as u32 ; const yuv410 : u32 = Format :: yuv410 as u32 ; const yvu410 : u32 = Format :: yvu410 as u32 ; const yuv411 : u32 = Format :: yuv411 as u32 ; const yvu411 : u32 = Format :: yvu411 as u32 ; const yuv420 : u32 = Format :: yuv420 as u32 ; const yvu420 : u32 = Format :: yvu420 as u32 ; const yuv422 : u32 = Format :: yuv422 as u32 ; const yvu422 : u32 = Format :: yvu422 as u32 ; const yuv444 : u32 = Format :: yuv444 as u32 ; const yvu444 : u32 = Format :: yvu444 as u32 ; const r8 : u32 = Format :: r8 as u32 ; const r16 : u32 = Format :: r16 as u32 ; const rg88 : u32 = Format :: rg88 as u32 ; const gr88 : u32 = Format :: gr88 as u32 ; const rg1616 : u32 = Format :: rg1616 as u32 ; const gr1616 : u32 = Format :: gr1616 as u32 ; const xrgb16161616f : u32 = Format :: xrgb16161616f as u32 ; const xbgr16161616f : u32 = Format :: xbgr16161616f as u32 ; const argb16161616f : u32 = Format :: argb16161616f as u32 ; const abgr16161616f : u32 = Format :: abgr16161616f as u32 ; const xyuv8888 : u32 = Format :: xyuv8888 as u32 ; const vuy888 : u32 = Format :: vuy888 as u32 ; const vuy101010 : u32 = Format :: vuy101010 as u32 ; const y210 : u32 = Format :: y210 as u32 ; const y212 : u32 = Format :: y212 as u32 ; const y216 : u32 = Format :: y216 as u32 ; const y410 : u32 = Format :: y410 as u32 ; const y412 : u32 = Format :: y412 as u32 ; const y416 : u32 = Format :: y416 as u32 ; const xvyu2101010 : u32 = Format :: xvyu2101010 as u32 ; const xvyu12_16161616 : u32 = Format :: xvyu12_16161616 as u32 ; const xvyu16161616 : u32 = Format :: xvyu16161616 as u32 ; const y0l0 : u32 = Format :: y0l0 as u32 ; const x0l0 : u32 = Format :: x0l0 as u32 ; const y0l2 : u32 = Format :: y0l2 as u32 ; const x0l2 : u32 = Format :: x0l2 as u32 ; const yuv420_8bit : u32 = Format :: yuv420_8bit as u32 ; const yuv420_10bit : u32 = Format :: yuv420_10bit as u32 ; const xrgb8888_a8 : u32 = Format :: xrgb8888_a8 as u32 ; const xbgr8888_a8 : u32 = Format :: xbgr8888_a8 as u32 ; const rgbx8888_a8 : u32 = Format :: rgbx8888_a8 as u32 ; const bgrx8888_a8 : u32 = Format :: bgrx8888_a8 as u32 ; const rgb888_a8 : u32 = Format :: rgb888_a8 as u32 ; const bgr888_a8 : u32 = Format :: bgr888_a8 as u32 ; const rgb565_a8 : u32 = Format :: rgb565_a8 as u32 ; const bgr565_a8 : u32 = Format :: bgr565_a8 as u32 ; const nv24 : u32 = Format :: nv24 as u32 ; const nv42 : u32 = Format :: nv42 as u32 ; const p210 : u32 = Format :: p210 as u32 ; const p010 : u32 = Format :: p010 as u32 ; const p012 : u32 = Format :: p012 as u32 ; const p016 : u32 = Format :: p016 as u32 ; const axbxgxrx106106106106 : u32 = Format :: axbxgxrx106106106106 as u32 ; const nv15 : u32 = Format :: nv15 as u32 ; const q410 : u32 = Format :: q410 as u32 ; const q401 : u32 = Format :: q401 as u32 ; const xrgb16161616 : u32 = Format :: xrgb16161616 as u32 ; const xbgr16161616 : u32 = Format :: xbgr16161616 as u32 ; const argb16161616 : u32 = Format :: argb16161616 as u32 ; const abgr16161616 : u32 = Format :: abgr16161616 as u32 ; const c1 : u32 = Format :: c1 as u32 ; const c2 : u32 = Format :: c2 as u32 ; const c4 : u32 = Format :: c4 as u32 ; const d1 : u32 = Format :: d1 as u32 ; const d2 : u32 = Format :: d2 as u32 ; const d4 : u32 = Format :: d4 as u32 ; const d8 : u32 = Format :: d8 as u32 ; const r1 : u32 = Format :: r1 as u32 ; const r2 : u32 = Format :: r2 as u32 ; const r4 : u32 = Format :: r4 as u32 ; const r10 : u32 = Format :: r10 as u32 ; const r12 : u32 = Format :: r12 as u32 ; const avuy8888 : u32 = Format :: avuy8888 as u32 ; const xvuy8888 : u32 = Format :: xvuy8888 as u32 ; const p030 : u32 = Format :: p030 as u32 ; match value { argb8888 => Ok (Self :: argb8888) , xrgb8888 => Ok (Self :: xrgb8888) , c8 => Ok (Self :: c8) , rgb332 => Ok (Self :: rgb332) , bgr233 => Ok (Self :: bgr233) , xrgb4444 => Ok (Self :: xrgb4444) , xbgr4444 => Ok (Self :: xbgr4444) , rgbx4444 => Ok (Self :: rgbx4444) , bgrx4444 => Ok (Self :: bgrx4444) , argb4444 => Ok (Self :: argb4444) , abgr4444 => Ok (Self :: abgr4444) , rgba4444 => Ok (Self :: rgba4444) , bgra4444 => Ok (Self :: bgra4444) , xrgb1555 => Ok (Self :: xrgb1555) , xbgr1555 => Ok (Self :: xbgr1555) , rgbx5551 => Ok (Self :: rgbx5551) , bgrx5551 => Ok (Self :: bgrx5551) , argb1555 => Ok (Self :: argb1555) , abgr1555 => Ok (Self :: abgr1555) , rgba5551 => Ok (Self :: rgba5551) , bgra5551 => Ok (Self :: bgra5551) , rgb565 => Ok (Self :: rgb565) , bgr565 => Ok (Self :: bgr565) , rgb888 => Ok (Self :: rgb888) , bgr888 => Ok (Self :: bgr888) , xbgr8888 => Ok (Self :: xbgr8888) , rgbx8888 => Ok (Self :: rgbx8888) , bgrx8888 => Ok (Self :: bgrx8888) , abgr8888 => Ok (Self :: abgr8888) , rgba8888 => Ok (Self :: rgba8888) , bgra8888 => Ok (Self :: bgra8888) , xrgb2101010 => Ok (Self :: xrgb2101010) , xbgr2101010 => Ok (Self :: xbgr2101010) , rgbx1010102 => Ok (Self :: rgbx1010102) , bgrx1010102 => Ok (Self :: bgrx1010102) , argb2101010 => Ok (Self :: argb2101010) , abgr2101010 => Ok (Self :: abgr2101010) , rgba1010102 => Ok (Self :: rgba1010102) , bgra1010102 => Ok (Self :: bgra1010102) , yuyv => Ok (Self :: yuyv) , yvyu => Ok (Self :: yvyu) , uyvy => Ok (Self :: uyvy) , vyuy => Ok (Self :: vyuy) , ayuv => Ok (Self :: ayuv) , nv12 => Ok (Self :: nv12) , nv21 => Ok (Self :: nv21) , nv16 => Ok (Self :: nv16) , nv61 => Ok (Self :: nv61) , yuv410 => Ok (Self :: yuv410) , yvu410 => Ok (Self :: yvu410) , yuv411 => Ok (Self :: yuv411) , yvu411 => Ok (Self :: yvu411) , yuv420 => Ok (Self :: yuv420) , yvu420 => Ok (Self :: yvu420) , yuv422 => Ok (Self :: yuv422) , yvu422 => Ok (Self :: yvu422) , yuv444 => Ok (Self :: yuv444) , yvu444 => Ok (Self :: yvu444) , r8 => Ok (Self :: r8) , r16 => Ok (Self :: r16) , rg88 => Ok (Self :: rg88) , gr88 => Ok (Self :: gr88) , rg1616 => Ok (Self :: rg1616) , gr1616 => Ok (Self :: gr1616) , xrgb16161616f => Ok (Self :: xrgb16161616f) , xbgr16161616f => Ok (Self :: xbgr16161616f) , argb16161616f => Ok (Self :: argb16161616f) , abgr16161616f => Ok (Self :: abgr16161616f) , xyuv8888 => Ok (Self :: xyuv8888) , vuy888 => Ok (Self :: vuy888) , vuy101010 => Ok (Self :: vuy101010) , y210 => Ok (Self :: y210) , y212 => Ok (Self :: y212) , y216 => Ok (Self :: y216) , y410 => Ok (Self :: y410) , y412 => Ok (Self :: y412) , y416 => Ok (Self :: y416) , xvyu2101010 => Ok (Self :: xvyu2101010) , xvyu12_16161616 => Ok (Self :: xvyu12_16161616) , xvyu16161616 => Ok (Self :: xvyu16161616) , y0l0 => Ok (Self :: y0l0) , x0l0 => Ok (Self :: x0l0) , y0l2 => Ok (Self :: y0l2) , x0l2 => Ok (Self :: x0l2) , yuv420_8bit => Ok (Self :: yuv420_8bit) , yuv420_10bit => Ok (Self :: yuv420_10bit) , xrgb8888_a8 => Ok (Self :: xrgb8888_a8) , xbgr8888_a8 => Ok (Self :: xbgr8888_a8) , rgbx8888_a8 => Ok (Self :: rgbx8888_a8) , bgrx8888_a8 => Ok (Self :: bgrx8888_a8) , rgb888_a8 => Ok (Self :: rgb888_a8) , bgr888_a8 => Ok (Self :: bgr888_a8) , rgb565_a8 => Ok (Self :: rgb565_a8) , bgr565_a8 => Ok (Self :: bgr565_a8) , nv24 => Ok (Self :: nv24) , nv42 => Ok (Self :: nv42) , p210 => Ok (Self :: p210) , p010 => Ok (Self :: p010) , p012 => Ok (Self :: p012) , p016 => Ok (Self :: p016) , axbxgxrx106106106106 => Ok (Self :: axbxgxrx106106106106) , nv15 => Ok (Self :: nv15) , q410 => Ok (Self :: q410) , q401 => Ok (Self :: q401) , xrgb16161616 => Ok (Self :: xrgb16161616) , xbgr16161616 => Ok (Self :: xbgr16161616) , argb16161616 => Ok (Self :: argb16161616) , abgr16161616 => Ok (Self :: abgr16161616) , c1 => Ok (Self :: c1) , c2 => Ok (Self :: c2) , c4 => Ok (Self :: c4) , d1 => Ok (Self :: d1) , d2 => Ok (Self :: d2) , d4 => Ok (Self :: d4) , d8 => Ok (Self :: d8) , r1 => Ok (Self :: r1) , r2 => Ok (Self :: r2) , r4 => Ok (Self :: r4) , r10 => Ok (Self :: r10) , r12 => Ok (Self :: r12) , avuy8888 => Ok (Self :: avuy8888) , xvuy8888 => Ok (Self :: xvuy8888) , p030 => Ok (Self :: p030) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Format" , otherwise))) , } } } impl std :: convert :: From < Format > for u32 { fn from (value : Format) -> u32 { value as u32 } } } # [doc = "content for a wl_surface\n\nA buffer provides the content for a wl_surface. Buffers are\ncreated through factory interfaces such as wl_shm, wp_linux_buffer_params\n(from the linux-dmabuf protocol extension) or similar. It has a width and\na height and can be attached to a wl_surface, but the mechanism by which a\nclient provides and updates the contents is defined by the buffer factory\ninterface.\n\nColor channels are assumed to be electrical rather than optical (in other\nwords, encoded with a transfer function) unless otherwise specified. If\nthe buffer uses a format that has an alpha channel, the alpha channel is\nassumed to be premultiplied into the electrical color channel values\n(after transfer function encoding) unless otherwise specified.\n\nNote, because wl_buffer objects are created from multiple independent\nfactory interfaces, the wl_buffer interface is frozen at version 1.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_buffer { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_buffer" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "compositor releases buffer\n\nSent when this wl_buffer is no longer used by the compositor.\nThe client is now free to reuse or destroy this buffer and its\nbacking storage.\n\nIf a client receives a release event before the frame callback\nrequested in the same wl_surface.commit that attaches this\nwl_buffer to a surface, then the client is immediately free to\nreuse the buffer and its backing storage, and does not need a\nsecond buffer for the next surface content update. Typically\nthis is possible, when the compositor maintains a copy of the\nwl_surface contents, e.g. as a GL texture. This is an important\noptimization for GL(ES) compositors with wl_shm clients.\n"] fn release (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { state . release (wire_msg . sender_id () ,) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_buffer" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "destroy a buffer\n\nDestroy a buffer. If and how you need to release the backing\nstorage is defined by the buffer factory interface.\n\nFor possible side-effects to a surface, see wl_surface.attach.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "offer to transfer data\n\nA wl_data_offer represents a piece of data offered for transfer\nby another client (the source client).  It is used by the\ncopy-and-paste and drag-and-drop mechanisms.  The offer\ndescribes the different mime types that the data can be\nconverted to and provides the mechanism for transferring the\ndata directly from the source client.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_data_offer { use super :: * ; pub const MAX_VERSION : u32 = 3u32 ; pub const NAME : & str = "wl_data_offer" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "advertise offered mime type\n\nSent immediately after creating the wl_data_offer object.  One\nevent per offered mime type.\n"] fn offer (& mut self , sender_id : waybackend :: types :: ObjectId , mime_type : & str) ; # [doc = "notify the source-side available actions\n\nThis event indicates the actions offered by the data source. It\nwill be sent immediately after creating the wl_data_offer object,\nor anytime the source side changes its offered actions through\nwl_data_source.set_actions.\n"] fn source_actions (& mut self , sender_id : waybackend :: types :: ObjectId , source_actions : wl_data_device_manager :: DndAction) ; # [doc = "notify the selected action\n\nThis event indicates the action selected by the compositor after\nmatching the source/destination side actions. Only one action (or\nnone) will be offered here.\n\nThis event can be emitted multiple times during the drag-and-drop\noperation in response to destination side action changes through\nwl_data_offer.set_actions.\n\nThis event will no longer be emitted after wl_data_device.drop\nhappened on the drag-and-drop destination, the client must\nhonor the last action received, or the last preferred one set\nthrough wl_data_offer.set_actions when handling an \"ask\" action.\n\nCompositors may also change the selected action on the fly, mainly\nin response to keyboard modifier changes during the drag-and-drop\noperation.\n\nThe most recent action received is always the valid one. Prior to\nreceiving wl_data_device.drop, the chosen action may change (e.g.\ndue to keyboard modifiers being pressed). At the time of receiving\nwl_data_device.drop the drag-and-drop destination must honor the\nlast action received.\n\nAction changes may still happen after wl_data_device.drop,\nespecially on \"ask\" actions, where the drag-and-drop destination\nmay choose another action afterwards. Action changes happening\nat this stage are always the result of inter-client negotiation, the\ncompositor shall no longer be able to induce a different action.\n\nUpon \"ask\" actions, it is expected that the drag-and-drop destination\nmay potentially choose a different action and/or mime type,\nbased on wl_data_offer.source_actions and finally chosen by the\nuser (e.g. popping up a menu with the available options). The\nfinal wl_data_offer.set_actions and wl_data_offer.accept requests\nmust happen before the call to wl_data_offer.finish.\n"] fn action (& mut self , sender_id : waybackend :: types :: ObjectId , dnd_action : wl_data_device_manager :: DndAction) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let mime_type = wire_msg . next_string () ? ; state . offer (wire_msg . sender_id () , mime_type) ; Ok (()) } , 1u16 => { let source_actions : wl_data_device_manager :: DndAction = wire_msg . next_u32 () . try_into () ? ; state . source_actions (wire_msg . sender_id () , source_actions) ; Ok (()) } , 2u16 => { let dnd_action : wl_data_device_manager :: DndAction = wire_msg . next_u32 () . try_into () ? ; state . action (wire_msg . sender_id () , dnd_action) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_data_offer" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const ACCEPT : u16 = 0u16 ; # [doc = "accept one of the offered mime types\n\nIndicate that the client can accept the given mime type, or\nNULL for not accepted.\n\nFor objects of version 2 or older, this request is used by the\nclient to give feedback whether the client can receive the given\nmime type, or NULL if none is accepted; the feedback does not\ndetermine whether the drag-and-drop operation succeeds or not.\n\nFor objects of version 3 or newer, this request determines the\nfinal result of the drag-and-drop operation. If the end result\nis that no mime types were accepted, the drag-and-drop operation\nwill be cancelled and the corresponding drag source will receive\nwl_data_source.cancelled. Clients may still use this event in\nconjunction with wl_data_source.action for feedback.\n"] pub fn accept (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , serial : u32 , mime_type : & str) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , ACCEPT) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . add_string (wayland_fd , mime_type) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RECEIVE : u16 = 1u16 ; # [doc = "request that the data is transferred\n\nTo transfer the offered data, the client issues this request\nand indicates the mime type it wants to receive.  The transfer\nhappens through the passed file descriptor (typically created\nwith the pipe system call).  The source client writes the data\nin the mime type representation requested and then closes the\nfile descriptor.\n\nThe receiving client reads from the read end of the pipe until\nEOF and then closes its end, at which point the transfer is\ncomplete.\n\nThis request may happen multiple times for different mime types,\nboth before and after wl_data_device.drop. Drag-and-drop destination\nclients may preemptively fetch data or examine it more closely to\ndetermine acceptance.\n"] pub fn receive (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , mime_type : & str , fd : & impl waybackend :: rustix :: fd :: AsRawFd) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RECEIVE) ? ; wire_msg_builder . add_string (wayland_fd , mime_type) ? ; wire_msg_builder . add_fd (wayland_fd , fd) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DESTROY : u16 = 2u16 ; # [doc = "destroy data offer\n\nDestroy the data offer.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const FINISH : u16 = 3u16 ; # [doc = "the offer will no longer be used\n\nNotifies the compositor that the drag destination successfully\nfinished the drag-and-drop operation.\n\nUpon receiving this request, the compositor will emit\nwl_data_source.dnd_finished on the drag source client.\n\nIt is a client error to perform other requests than\nwl_data_offer.destroy after this one. It is also an error to perform\nthis request after a NULL mime type has been set in\nwl_data_offer.accept or no action was received through\nwl_data_offer.action.\n\nIf wl_data_offer.finish request is received for a non drag and drop\noperation, the invalid_finish protocol error is raised.\n"] pub fn finish (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , FINISH) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_ACTIONS : u16 = 4u16 ; # [doc = "set the available/preferred drag-and-drop actions\n\nSets the actions that the destination side client supports for\nthis operation. This request may trigger the emission of\nwl_data_source.action and wl_data_offer.action events if the compositor\nneeds to change the selected action.\n\nThis request can be called multiple times throughout the\ndrag-and-drop operation, typically in response to wl_data_device.enter\nor wl_data_device.motion events.\n\nThis request determines the final result of the drag-and-drop\noperation. If the end result is that no action is accepted,\nthe drag source will receive wl_data_source.cancelled.\n\nThe dnd_actions argument must contain only values expressed in the\nwl_data_device_manager.dnd_actions enum, and the preferred_action\nargument must only contain one of those values set, otherwise it\nwill result in a protocol error.\n\nWhile managing an \"ask\" action, the destination drag-and-drop client\nmay perform further wl_data_offer.receive requests, and is expected\nto perform one last wl_data_offer.set_actions request with a preferred\naction other than \"ask\" (and optionally wl_data_offer.accept) before\nrequesting wl_data_offer.finish, in order to convey the action selected\nby the user. If the preferred action is not in the\nwl_data_offer.source_actions mask, an error will be raised.\n\nIf the \"ask\" action is dismissed (e.g. user cancellation), the client\nis expected to perform wl_data_offer.destroy right away.\n\nThis request can only be made on drag-and-drop offers, a protocol error\nwill be raised otherwise.\n"] pub fn set_actions (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , dnd_actions : wl_data_device_manager :: DndAction , preferred_action : wl_data_device_manager :: DndAction) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_ACTIONS) ? ; wire_msg_builder . add_u32 (wayland_fd , dnd_actions . into ()) ? ; wire_msg_builder . add_u32 (wayland_fd , preferred_action . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "finish request was called untimely"] invalid_finish = 0u32 , # [doc = "action mask contains invalid values"] invalid_action_mask = 1u32 , # [doc = "action argument has an invalid value"] invalid_action = 2u32 , # [doc = "offer doesn't accept this request"] invalid_offer = 3u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_finish : u32 = Error :: invalid_finish as u32 ; const invalid_action_mask : u32 = Error :: invalid_action_mask as u32 ; const invalid_action : u32 = Error :: invalid_action as u32 ; const invalid_offer : u32 = Error :: invalid_offer as u32 ; match value { invalid_finish => Ok (Self :: invalid_finish) , invalid_action_mask => Ok (Self :: invalid_action_mask) , invalid_action => Ok (Self :: invalid_action) , invalid_offer => Ok (Self :: invalid_offer) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "offer to transfer data\n\nThe wl_data_source object is the source side of a wl_data_offer.\nIt is created by the source client in a data transfer and\nprovides a way to describe the offered data and a way to respond\nto requests to transfer the data.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_data_source { use super :: * ; pub const MAX_VERSION : u32 = 3u32 ; pub const NAME : & str = "wl_data_source" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "a target accepts an offered mime type\n\nSent when a target accepts pointer_focus or motion events.  If\na target does not accept any of the offered types, type is NULL.\n\nUsed for feedback during drag-and-drop.\n"] fn target (& mut self , sender_id : waybackend :: types :: ObjectId , mime_type : & str) ; # [doc = "send the data\n\nRequest for data from the client.  Send the data as the\nspecified mime type over the passed file descriptor, then\nclose it.\n"] fn send (& mut self , sender_id : waybackend :: types :: ObjectId , mime_type : & str , fd : & impl waybackend :: rustix :: fd :: AsRawFd) ; # [doc = "selection was cancelled\n\nThis data source is no longer valid. There are several reasons why\nthis could happen:\n\n- The data source has been replaced by another data source.\n- The drag-and-drop operation was performed, but the drop destination\ndid not accept any of the mime types offered through\nwl_data_source.target.\n- The drag-and-drop operation was performed, but the drop destination\ndid not select any of the actions present in the mask offered through\nwl_data_source.action.\n- The drag-and-drop operation was performed but didn't happen over a\nsurface.\n- The compositor cancelled the drag-and-drop operation (e.g. compositor\ndependent timeouts to avoid stale drag-and-drop transfers).\n\nThe client should clean up and destroy this data source.\n\nFor objects of version 2 or older, wl_data_source.cancelled will\nonly be emitted if the data source was replaced by another data\nsource.\n"] fn cancelled (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "the drag-and-drop operation physically finished\n\nThe user performed the drop action. This event does not indicate\nacceptance, wl_data_source.cancelled may still be emitted afterwards\nif the drop destination does not accept any mime type.\n\nHowever, this event might however not be received if the compositor\ncancelled the drag-and-drop operation before this event could happen.\n\nNote that the data_source may still be used in the future and should\nnot be destroyed here.\n"] fn dnd_drop_performed (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "the drag-and-drop operation concluded\n\nThe drop destination finished interoperating with this data\nsource, so the client is now free to destroy this data source and\nfree all associated data.\n\nIf the action used to perform the operation was \"move\", the\nsource can now delete the transferred data.\n"] fn dnd_finished (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "notify the selected action\n\nThis event indicates the action selected by the compositor after\nmatching the source/destination side actions. Only one action (or\nnone) will be offered here.\n\nThis event can be emitted multiple times during the drag-and-drop\noperation, mainly in response to destination side changes through\nwl_data_offer.set_actions, and as the data device enters/leaves\nsurfaces.\n\nIt is only possible to receive this event after\nwl_data_source.dnd_drop_performed if the drag-and-drop operation\nended in an \"ask\" action, in which case the final wl_data_source.action\nevent will happen immediately before wl_data_source.dnd_finished.\n\nCompositors may also change the selected action on the fly, mainly\nin response to keyboard modifier changes during the drag-and-drop\noperation.\n\nThe most recent action received is always the valid one. The chosen\naction may change alongside negotiation (e.g. an \"ask\" action can turn\ninto a \"move\" operation), so the effects of the final action must\nalways be applied in wl_data_offer.dnd_finished.\n\nClients can trigger cursor surface changes from this point, so\nthey reflect the current action.\n"] fn action (& mut self , sender_id : waybackend :: types :: ObjectId , dnd_action : wl_data_device_manager :: DndAction) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let mime_type = wire_msg . next_string () ? ; state . target (wire_msg . sender_id () , mime_type) ; Ok (()) } , 1u16 => { let mime_type = wire_msg . next_string () ? ; let fd = wire_msg . next_fd () ; let fd = & fd ; state . send (wire_msg . sender_id () , mime_type , fd) ; Ok (()) } , 2u16 => { state . cancelled (wire_msg . sender_id () ,) ; Ok (()) } , 3u16 => { state . dnd_drop_performed (wire_msg . sender_id () ,) ; Ok (()) } , 4u16 => { state . dnd_finished (wire_msg . sender_id () ,) ; Ok (()) } , 5u16 => { let dnd_action : wl_data_device_manager :: DndAction = wire_msg . next_u32 () . try_into () ? ; state . action (wire_msg . sender_id () , dnd_action) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_data_source" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const OFFER : u16 = 0u16 ; # [doc = "add an offered mime type\n\nThis request adds a mime type to the set of mime types\nadvertised to targets.  Can be called several times to offer\nmultiple types.\n"] pub fn offer (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , mime_type : & str) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , OFFER) ? ; wire_msg_builder . add_string (wayland_fd , mime_type) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DESTROY : u16 = 1u16 ; # [doc = "destroy the data source\n\nDestroy the data source.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_ACTIONS : u16 = 2u16 ; # [doc = "set the available drag-and-drop actions\n\nSets the actions that the source side client supports for this\noperation. This request may trigger wl_data_source.action and\nwl_data_offer.action events if the compositor needs to change the\nselected action.\n\nThe dnd_actions argument must contain only values expressed in the\nwl_data_device_manager.dnd_actions enum, otherwise it will result\nin a protocol error.\n\nThis request must be made once only, and can only be made on sources\nused in drag-and-drop, so it must be performed before\nwl_data_device.start_drag. Attempting to use the source other than\nfor drag-and-drop will raise a protocol error.\n"] pub fn set_actions (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , dnd_actions : wl_data_device_manager :: DndAction) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_ACTIONS) ? ; wire_msg_builder . add_u32 (wayland_fd , dnd_actions . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "action mask contains invalid values"] invalid_action_mask = 0u32 , # [doc = "source doesn't accept this request"] invalid_source = 1u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_action_mask : u32 = Error :: invalid_action_mask as u32 ; const invalid_source : u32 = Error :: invalid_source as u32 ; match value { invalid_action_mask => Ok (Self :: invalid_action_mask) , invalid_source => Ok (Self :: invalid_source) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "data transfer device\n\nThere is one wl_data_device per seat which can be obtained\nfrom the global wl_data_device_manager singleton.\n\nA wl_data_device provides access to inter-client data transfer\nmechanisms such as copy-and-paste and drag-and-drop.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_data_device { use super :: * ; pub const MAX_VERSION : u32 = 3u32 ; pub const NAME : & str = "wl_data_device" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "introduce a new wl_data_offer\n\nThe data_offer event introduces a new wl_data_offer object,\nwhich will subsequently be used in either the\ndata_device.enter event (for drag-and-drop) or the\ndata_device.selection event (for selections).  Immediately\nfollowing the data_device.data_offer event, the new data_offer\nobject will send out data_offer.offer events to describe the\nmime types it offers.\n"] fn data_offer (& mut self , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) ; # [doc = "initiate drag-and-drop session\n\nThis event is sent when an active drag-and-drop pointer enters\na surface owned by the client.  The position of the pointer at\nenter time is provided by the x and y arguments, in surface-local\ncoordinates.\n"] fn enter (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : waybackend :: types :: ObjectId , x : waybackend :: types :: WlFixed , y : waybackend :: types :: WlFixed , id : Option < waybackend :: types :: ObjectId >) ; # [doc = "end drag-and-drop session\n\nThis event is sent when the drag-and-drop pointer leaves the\nsurface and the session ends.  The client must destroy the\nwl_data_offer introduced at enter time at this point.\n"] fn leave (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "drag-and-drop session motion\n\nThis event is sent when the drag-and-drop pointer moves within\nthe currently focused surface. The new position of the pointer\nis provided by the x and y arguments, in surface-local\ncoordinates.\n"] fn motion (& mut self , sender_id : waybackend :: types :: ObjectId , time : u32 , x : waybackend :: types :: WlFixed , y : waybackend :: types :: WlFixed) ; # [doc = "end drag-and-drop session successfully\n\nThe event is sent when a drag-and-drop operation is ended\nbecause the implicit grab is removed.\n\nThe drag-and-drop destination is expected to honor the last action\nreceived through wl_data_offer.action, if the resulting action is\n\"copy\" or \"move\", the destination can still perform\nwl_data_offer.receive requests, and is expected to end all\ntransfers with a wl_data_offer.finish request.\n\nIf the resulting action is \"ask\", the action will not be considered\nfinal. The drag-and-drop destination is expected to perform one last\nwl_data_offer.set_actions request, or wl_data_offer.destroy in order\nto cancel the operation.\n"] fn drop (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "advertise new selection\n\nThe selection event is sent out to notify the client of a new\nwl_data_offer for the selection for this device.  The\ndata_device.data_offer and the data_offer.offer events are\nsent out immediately before this event to introduce the data\noffer object.  The selection event is sent to a client\nimmediately before receiving keyboard focus and when a new\nselection is set while the client has keyboard focus.  The\ndata_offer is valid until a new data_offer or NULL is received\nor until the client loses keyboard focus.  Switching surface with\nkeyboard focus within the same client doesn't mean a new selection\nwill be sent.  The client must destroy the previous selection\ndata_offer, if any, upon receiving this event.\n"] fn selection (& mut self , sender_id : waybackend :: types :: ObjectId , id : Option < waybackend :: types :: ObjectId >) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let id = wire_msg . next_new_specified_id () ? ; state . data_offer (wire_msg . sender_id () , id) ; Ok (()) } , 1u16 => { let serial = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; let x = wire_msg . next_fixed () ; let y = wire_msg . next_fixed () ; let id = wire_msg . next_object () ; state . enter (wire_msg . sender_id () , serial , surface , x , y , id) ; Ok (()) } , 2u16 => { state . leave (wire_msg . sender_id () ,) ; Ok (()) } , 3u16 => { let time = wire_msg . next_u32 () ; let x = wire_msg . next_fixed () ; let y = wire_msg . next_fixed () ; state . motion (wire_msg . sender_id () , time , x , y) ; Ok (()) } , 4u16 => { state . drop (wire_msg . sender_id () ,) ; Ok (()) } , 5u16 => { let id = wire_msg . next_object () ; state . selection (wire_msg . sender_id () , id) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_data_device" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const START_DRAG : u16 = 0u16 ; # [doc = "start drag-and-drop operation\n\nThis request asks the compositor to start a drag-and-drop\noperation on behalf of the client.\n\nThe source argument is the data source that provides the data\nfor the eventual data transfer. If source is NULL, enter, leave\nand motion events are sent only to the client that initiated the\ndrag and the client is expected to handle the data passing\ninternally. If source is destroyed, the drag-and-drop session will be\ncancelled.\n\nThe origin surface is the surface where the drag originates and\nthe client must have an active implicit grab that matches the\nserial.\n\nThe icon surface is an optional (can be NULL) surface that\nprovides an icon to be moved around with the cursor.  Initially,\nthe top-left corner of the icon surface is placed at the cursor\nhotspot, but subsequent wl_surface.offset requests can move the\nrelative position. Attach requests must be confirmed with\nwl_surface.commit as usual. The icon surface is given the role of\na drag-and-drop icon. If the icon surface already has another role,\nit raises a protocol error.\n\nThe input region is ignored for wl_surfaces with the role of a\ndrag-and-drop icon.\n\nThe given source may not be used in any further set_selection or\nstart_drag requests. Attempting to reuse a previously-used source\nmay send a used_source error.\n"] pub fn start_drag (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , source : Option < waybackend :: types :: ObjectId > , origin : waybackend :: types :: ObjectId , icon : Option < waybackend :: types :: ObjectId > , serial : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , START_DRAG) ? ; wire_msg_builder . add_object (wayland_fd , source) ? ; wire_msg_builder . add_object (wayland_fd , Some (origin)) ? ; wire_msg_builder . add_object (wayland_fd , icon) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_SELECTION : u16 = 1u16 ; # [doc = "copy data to the selection\n\nThis request asks the compositor to set the selection\nto the data from the source on behalf of the client.\n\nTo unset the selection, set the source to NULL.\n\nThe given source may not be used in any further set_selection or\nstart_drag requests. Attempting to reuse a previously-used source\nmay send a used_source error.\n"] pub fn set_selection (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , source : Option < waybackend :: types :: ObjectId > , serial : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_SELECTION) ? ; wire_msg_builder . add_object (wayland_fd , source) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RELEASE : u16 = 2u16 ; # [doc = "destroy data device\n\nThis request destroys the data device.\n\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "given wl_surface has another role"] role = 0u32 , # [doc = "source has already been used"] used_source = 1u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const role : u32 = Error :: role as u32 ; const used_source : u32 = Error :: used_source as u32 ; match value { role => Ok (Self :: role) , used_source => Ok (Self :: used_source) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "data transfer interface\n\nThe wl_data_device_manager is a singleton global object that\nprovides access to inter-client data transfer mechanisms such as\ncopy-and-paste and drag-and-drop.  These mechanisms are tied to\na wl_seat and this interface lets a client get a wl_data_device\ncorresponding to a wl_seat.\n\nDepending on the version bound, the objects created from the bound\nwl_data_device_manager object will have different requirements for\nfunctioning properly. See wl_data_source.set_actions,\nwl_data_offer.accept and wl_data_offer.finish for details.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_data_device_manager { use super :: * ; pub const MAX_VERSION : u32 = 3u32 ; pub const NAME : & str = "wl_data_device_manager" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_data_device_manager" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const CREATE_DATA_SOURCE : u16 = 0u16 ; # [doc = "create a new data source\n\nCreate a new data source.\n"] pub fn create_data_source (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , CREATE_DATA_SOURCE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_DATA_DEVICE : u16 = 1u16 ; # [doc = "create a new data device\n\nCreate a new data device for a given seat.\n"] pub fn get_data_device (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , seat : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_DATA_DEVICE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (seat)) ? ; wire_msg_builder . finish () ; Ok (()) } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = "drag and drop actions\n\nThis is a bitmask of the available/preferred actions in a\ndrag-and-drop operation.\n\nIn the compositor, the selected action is a result of matching the\nactions offered by the source and destination sides.  \"action\" events\nwith a \"none\" action will be sent to both source and destination if\nthere is no match. All further checks will effectively happen on\n(source actions ∩ destination actions).\n\nIn addition, compositors may also pick different actions in\nreaction to key modifiers being pressed. One common design that\nis used in major toolkits (and the behavior recommended for\ncompositors) is:\n\n- If no modifiers are pressed, the first match (in bit order)\nwill be used.\n- Pressing Shift selects \"move\", if enabled in the mask.\n- Pressing Control selects \"copy\", if enabled in the mask.\n\nBehavior beyond that is considered implementation-dependent.\nCompositors may for example bind other modifiers (like Alt/Meta)\nor drags initiated with other buttons than BTN_LEFT to specific\nactions (e.g. \"ask\").\n"] pub struct DndAction : u32 { # [doc = "no action"] const NONE = 0u32 ; # [doc = "copy action"] const COPY = 1u32 ; # [doc = "move action"] const MOVE = 2u32 ; # [doc = "ask action"] const ASK = 4u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for DndAction { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < DndAction > for u32 { fn from (value : DndAction) -> u32 { value . bits () } } } # [doc = "create desktop-style surfaces\n\nThis interface is implemented by servers that provide\ndesktop-style user interfaces.\n\nIt allows clients to associate a wl_shell_surface with\na basic surface.\n\nNote! This protocol is deprecated and not intended for production use.\nFor desktop-style user interfaces, use xdg_shell. Compositors and clients\nshould not implement this interface.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_shell { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_shell" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_shell" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const GET_SHELL_SURFACE : u16 = 0u16 ; # [doc = "create a shell surface from a surface\n\nCreate a shell surface for an existing surface. This gives\nthe wl_surface the role of a shell surface. If the wl_surface\nalready has another role, it raises a protocol error.\n\nOnly one shell surface can be associated with a given surface.\n"] pub fn get_shell_surface (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , surface : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_SHELL_SURFACE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (surface)) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "given wl_surface has another role"] role = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const role : u32 = Error :: role as u32 ; match value { role => Ok (Self :: role) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "desktop-style metadata interface\n\nAn interface that may be implemented by a wl_surface, for\nimplementations that provide a desktop-style user interface.\n\nIt provides requests to treat surfaces like toplevel, fullscreen\nor popup windows, move, resize or maximize them, associate\nmetadata like title and class, etc.\n\nOn the server side the object is automatically destroyed when\nthe related wl_surface is destroyed. On the client side,\nwl_shell_surface_destroy() must be called before destroying\nthe wl_surface object.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_shell_surface { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_shell_surface" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "ping client\n\nPing a client to check if it is receiving events and sending\nrequests. A client is expected to reply with a pong request.\n"] fn ping (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32) ; # [doc = "suggest resize\n\nThe configure event asks the client to resize its surface.\n\nThe size is a hint, in the sense that the client is free to\nignore it if it doesn't resize, pick a smaller size (to\nsatisfy aspect ratio or resize in steps of NxM pixels).\n\nThe edges parameter provides a hint about how the surface\nwas resized. The client may use this information to decide\nhow to adjust its content to the new size (e.g. a scrolling\narea might adjust its content position to leave the viewable\ncontent unmoved).\n\nThe client is free to dismiss all but the last configure\nevent it received.\n\nThe width and height arguments specify the size of the window\nin surface-local coordinates.\n"] fn configure (& mut self , sender_id : waybackend :: types :: ObjectId , edges : Resize , width : i32 , height : i32) ; # [doc = "popup interaction is done\n\nThe popup_done event is sent out when a popup grab is broken,\nthat is, when the user clicks a surface that doesn't belong\nto the client owning the popup surface.\n"] fn popup_done (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let serial = wire_msg . next_u32 () ; state . ping (wire_msg . sender_id () , serial) ; Ok (()) } , 1u16 => { let edges : Resize = wire_msg . next_u32 () . try_into () ? ; let width = wire_msg . next_i32 () ; let height = wire_msg . next_i32 () ; state . configure (wire_msg . sender_id () , edges , width , height) ; Ok (()) } , 2u16 => { state . popup_done (wire_msg . sender_id () ,) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_shell_surface" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const PONG : u16 = 0u16 ; # [doc = "respond to a ping event\n\nA client must respond to a ping event with a pong request or\nthe client may be deemed unresponsive.\n"] pub fn pong (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , serial : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , PONG) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . finish () ; Ok (()) } pub const MOVE : u16 = 1u16 ; # [doc = "start an interactive move\n\nStart a pointer-driven move of the surface.\n\nThis request must be used in response to a button press event.\nThe server may ignore move requests depending on the state of\nthe surface (e.g. fullscreen or maximized).\n"] pub fn _move (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , seat : waybackend :: types :: ObjectId , serial : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , MOVE) ? ; wire_msg_builder . add_object (wayland_fd , Some (seat)) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RESIZE : u16 = 2u16 ; # [doc = "start an interactive resize\n\nStart a pointer-driven resizing of the surface.\n\nThis request must be used in response to a button press event.\nThe server may ignore resize requests depending on the state of\nthe surface (e.g. fullscreen or maximized).\n"] pub fn resize (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , seat : waybackend :: types :: ObjectId , serial : u32 , edges : Resize) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RESIZE) ? ; wire_msg_builder . add_object (wayland_fd , Some (seat)) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . add_u32 (wayland_fd , edges . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_TOPLEVEL : u16 = 3u16 ; # [doc = "make the surface a toplevel surface\n\nMap the surface as a toplevel surface.\n\nA toplevel surface is not fullscreen, maximized or transient.\n"] pub fn set_toplevel (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_TOPLEVEL) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_TRANSIENT : u16 = 4u16 ; # [doc = "make the surface a transient surface\n\nMap the surface relative to an existing surface.\n\nThe x and y arguments specify the location of the upper left\ncorner of the surface relative to the upper left corner of the\nparent surface, in surface-local coordinates.\n\nThe flags argument controls details of the transient behaviour.\n"] pub fn set_transient (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , parent : waybackend :: types :: ObjectId , x : i32 , y : i32 , flags : Transient) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_TRANSIENT) ? ; wire_msg_builder . add_object (wayland_fd , Some (parent)) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_u32 (wayland_fd , flags . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_FULLSCREEN : u16 = 5u16 ; # [doc = "make the surface a fullscreen surface\n\nMap the surface as a fullscreen surface.\n\nIf an output parameter is given then the surface will be made\nfullscreen on that output. If the client does not specify the\noutput then the compositor will apply its policy - usually\nchoosing the output on which the surface has the biggest surface\narea.\n\nThe client may specify a method to resolve a size conflict\nbetween the output size and the surface size - this is provided\nthrough the method parameter.\n\nThe framerate parameter is used only when the method is set\nto \"driver\", to indicate the preferred framerate. A value of 0\nindicates that the client does not care about framerate.  The\nframerate is specified in mHz, that is framerate of 60000 is 60Hz.\n\nA method of \"scale\" or \"driver\" implies a scaling operation of\nthe surface, either via a direct scaling operation or a change of\nthe output mode. This will override any kind of output scaling, so\nthat mapping a surface with a buffer size equal to the mode can\nfill the screen independent of buffer_scale.\n\nA method of \"fill\" means we don't scale up the buffer, however\nany output scale is applied. This means that you may run into\nan edge case where the application maps a buffer with the same\nsize of the output mode but buffer_scale 1 (thus making a\nsurface larger than the output). In this case it is allowed to\ndownscale the results to fit the screen.\n\nThe compositor must reply to this request with a configure event\nwith the dimensions for the output on which the surface will\nbe made fullscreen.\n"] pub fn set_fullscreen (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , method : FullscreenMethod , framerate : u32 , output : Option < waybackend :: types :: ObjectId >) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_FULLSCREEN) ? ; wire_msg_builder . add_u32 (wayland_fd , method . into ()) ? ; wire_msg_builder . add_u32 (wayland_fd , framerate) ? ; wire_msg_builder . add_object (wayland_fd , output) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_POPUP : u16 = 6u16 ; # [doc = "make the surface a popup surface\n\nMap the surface as a popup.\n\nA popup surface is a transient surface with an added pointer\ngrab.\n\nAn existing implicit grab will be changed to owner-events mode,\nand the popup grab will continue after the implicit grab ends\n(i.e. releasing the mouse button does not cause the popup to\nbe unmapped).\n\nThe popup grab continues until the window is destroyed or a\nmouse button is pressed in any other client's window. A click\nin any of the client's surfaces is reported as normal, however,\nclicks in other clients' surfaces will be discarded and trigger\nthe callback.\n\nThe x and y arguments specify the location of the upper left\ncorner of the surface relative to the upper left corner of the\nparent surface, in surface-local coordinates.\n"] pub fn set_popup (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , seat : waybackend :: types :: ObjectId , serial : u32 , parent : waybackend :: types :: ObjectId , x : i32 , y : i32 , flags : Transient) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_POPUP) ? ; wire_msg_builder . add_object (wayland_fd , Some (seat)) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . add_object (wayland_fd , Some (parent)) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_u32 (wayland_fd , flags . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_MAXIMIZED : u16 = 7u16 ; # [doc = "make the surface a maximized surface\n\nMap the surface as a maximized surface.\n\nIf an output parameter is given then the surface will be\nmaximized on that output. If the client does not specify the\noutput then the compositor will apply its policy - usually\nchoosing the output on which the surface has the biggest surface\narea.\n\nThe compositor will reply with a configure event telling\nthe expected new surface size. The operation is completed\non the next buffer attach to this surface.\n\nA maximized surface typically fills the entire output it is\nbound to, except for desktop elements such as panels. This is\nthe main difference between a maximized shell surface and a\nfullscreen shell surface.\n\nThe details depend on the compositor implementation.\n"] pub fn set_maximized (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , output : Option < waybackend :: types :: ObjectId >) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_MAXIMIZED) ? ; wire_msg_builder . add_object (wayland_fd , output) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_TITLE : u16 = 8u16 ; # [doc = "set surface title\n\nSet a short title for the surface.\n\nThis string may be used to identify the surface in a task bar,\nwindow list, or other user interface elements provided by the\ncompositor.\n\nThe string must be encoded in UTF-8.\n"] pub fn set_title (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , title : & str) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_TITLE) ? ; wire_msg_builder . add_string (wayland_fd , title) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_CLASS : u16 = 9u16 ; # [doc = "set surface class\n\nSet a class for the surface.\n\nThe surface class identifies the general class of applications\nto which the surface belongs. A common convention is to use the\nfile name (or the full path if it is a non-standard location) of\nthe application's .desktop file as the class.\n"] pub fn set_class (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , class_ : & str) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_CLASS) ? ; wire_msg_builder . add_string (wayland_fd , class_) ? ; wire_msg_builder . finish () ; Ok (()) } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = "edge values for resizing\n\nThese values are used to indicate which edge of a surface\nis being dragged in a resize operation. The server may\nuse this information to adapt its behavior, e.g. choose\nan appropriate cursor image.\n"] pub struct Resize : u32 { # [doc = "no edge"] const NONE = 0u32 ; # [doc = "top edge"] const TOP = 1u32 ; # [doc = "bottom edge"] const BOTTOM = 2u32 ; # [doc = "left edge"] const LEFT = 4u32 ; # [doc = "top and left edges"] const TOP_LEFT = 5u32 ; # [doc = "bottom and left edges"] const BOTTOM_LEFT = 6u32 ; # [doc = "right edge"] const RIGHT = 8u32 ; # [doc = "top and right edges"] const TOP_RIGHT = 9u32 ; # [doc = "bottom and right edges"] const BOTTOM_RIGHT = 10u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for Resize { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < Resize > for u32 { fn from (value : Resize) -> u32 { value . bits () } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = "details of transient behaviour\n\nThese flags specify details of the expected behaviour\nof transient surfaces. Used in the set_transient request.\n"] pub struct Transient : u32 { # [doc = "do not set keyboard focus"] const INACTIVE = 1u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for Transient { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < Transient > for u32 { fn from (value : Transient) -> u32 { value . bits () } } # [doc = "different method to set the surface fullscreen\n\nHints to indicate to the compositor how to deal with a conflict\nbetween the dimensions of the surface and the dimensions of the\noutput. The compositor is free to ignore this parameter.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum FullscreenMethod { # [doc = "no preference, apply default policy"] default = 0u32 , # [doc = "scale, preserve the surface's aspect ratio and center on output"] scale = 1u32 , # [doc = "switch output mode to the smallest mode that can fit the surface, add black borders to compensate size mismatch"] driver = 2u32 , # [doc = "no upscaling, center on output and add black borders to compensate size mismatch"] fill = 3u32 , } impl std :: convert :: TryFrom < u32 > for FullscreenMethod { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const default : u32 = FullscreenMethod :: default as u32 ; const scale : u32 = FullscreenMethod :: scale as u32 ; const driver : u32 = FullscreenMethod :: driver as u32 ; const fill : u32 = FullscreenMethod :: fill as u32 ; match value { default => Ok (Self :: default) , scale => Ok (Self :: scale) , driver => Ok (Self :: driver) , fill => Ok (Self :: fill) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("FullscreenMethod" , otherwise))) , } } } impl std :: convert :: From < FullscreenMethod > for u32 { fn from (value : FullscreenMethod) -> u32 { value as u32 } } } # [doc = "an onscreen surface\n\nA surface is a rectangular area that may be displayed on zero\nor more outputs, and shown any number of times at the compositor's\ndiscretion. They can present wl_buffers, receive user input, and\ndefine a local coordinate system.\n\nThe size of a surface (and relative positions on it) is described\nin surface-local coordinates, which may differ from the buffer\ncoordinates of the pixel content, in case a buffer_transform\nor a buffer_scale is used.\n\nA surface without a \"role\" is fairly useless: a compositor does\nnot know where, when or how to present it. The role is the\npurpose of a wl_surface. Examples of roles are a cursor for a\npointer (as set by wl_pointer.set_cursor), a drag icon\n(wl_data_device.start_drag), a sub-surface\n(wl_subcompositor.get_subsurface), and a window as defined by a\nshell protocol (e.g. wl_shell.get_shell_surface).\n\nA surface can have only one role at a time. Initially a\nwl_surface does not have a role. Once a wl_surface is given a\nrole, it is set permanently for the whole lifetime of the\nwl_surface object. Giving the current role again is allowed,\nunless explicitly forbidden by the relevant interface\nspecification.\n\nSurface roles are given by requests in other interfaces such as\nwl_pointer.set_cursor. The request should explicitly mention\nthat this request gives a role to a wl_surface. Often, this\nrequest also creates a new protocol object that represents the\nrole and adds additional functionality to wl_surface. When a\nclient wants to destroy a wl_surface, they must destroy this role\nobject before the wl_surface, otherwise a defunct_role_object error is\nsent.\n\nDestroying the role object does not remove the role from the\nwl_surface, but it may stop the wl_surface from \"playing the role\".\nFor instance, if a wl_subsurface object is destroyed, the wl_surface\nit was created for will be unmapped and forget its position and\nz-order. It is allowed to create a wl_subsurface for the same\nwl_surface again, but it is not allowed to use the wl_surface as\na cursor (cursor is a different role than sub-surface, and role\nswitching is not allowed).\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_surface { use super :: * ; pub const MAX_VERSION : u32 = 6u32 ; pub const NAME : & str = "wl_surface" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "surface enters an output\n\nThis is emitted whenever a surface's creation, movement, or resizing\nresults in some part of it being within the scanout region of an\noutput.\n\nNote that a surface may be overlapping with zero or more outputs.\n"] fn enter (& mut self , sender_id : waybackend :: types :: ObjectId , output : waybackend :: types :: ObjectId) ; # [doc = "surface leaves an output\n\nThis is emitted whenever a surface's creation, movement, or resizing\nresults in it no longer having any part of it within the scanout region\nof an output.\n\nClients should not use the number of outputs the surface is on for frame\nthrottling purposes. The surface might be hidden even if no leave event\nhas been sent, and the compositor might expect new surface content\nupdates even if no enter event has been sent. The frame event should be\nused instead.\n"] fn leave (& mut self , sender_id : waybackend :: types :: ObjectId , output : waybackend :: types :: ObjectId) ; # [doc = "preferred buffer scale for the surface\n\nThis event indicates the preferred buffer scale for this surface. It is\nsent whenever the compositor's preference changes.\n\nBefore receiving this event the preferred buffer scale for this surface\nis 1.\n\nIt is intended that scaling aware clients use this event to scale their\ncontent and use wl_surface.set_buffer_scale to indicate the scale they\nhave rendered with. This allows clients to supply a higher detail\nbuffer.\n\nThe compositor shall emit a scale value greater than 0.\n"] fn preferred_buffer_scale (& mut self , sender_id : waybackend :: types :: ObjectId , factor : i32) ; # [doc = "preferred buffer transform for the surface\n\nThis event indicates the preferred buffer transform for this surface.\nIt is sent whenever the compositor's preference changes.\n\nBefore receiving this event the preferred buffer transform for this\nsurface is normal.\n\nApplying this transformation to the surface buffer contents and using\nwl_surface.set_buffer_transform might allow the compositor to use the\nsurface buffer more efficiently.\n"] fn preferred_buffer_transform (& mut self , sender_id : waybackend :: types :: ObjectId , transform : wl_output :: Transform) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let output = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; state . enter (wire_msg . sender_id () , output) ; Ok (()) } , 1u16 => { let output = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; state . leave (wire_msg . sender_id () , output) ; Ok (()) } , 2u16 => { let factor = wire_msg . next_i32 () ; state . preferred_buffer_scale (wire_msg . sender_id () , factor) ; Ok (()) } , 3u16 => { let transform : wl_output :: Transform = wire_msg . next_u32 () . try_into () ? ; state . preferred_buffer_transform (wire_msg . sender_id () , transform) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_surface" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "delete surface\n\nDeletes the surface and invalidates its object ID.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const ATTACH : u16 = 1u16 ; # [doc = "set the surface contents\n\nSet a buffer as the content of this surface.\n\nThe new size of the surface is calculated based on the buffer\nsize transformed by the inverse buffer_transform and the\ninverse buffer_scale. This means that at commit time the supplied\nbuffer size must be an integer multiple of the buffer_scale. If\nthat's not the case, an invalid_size error is sent.\n\nThe x and y arguments specify the location of the new pending\nbuffer's upper left corner, relative to the current buffer's upper\nleft corner, in surface-local coordinates. In other words, the\nx and y, combined with the new surface size define in which\ndirections the surface's size changes. Setting anything other than 0\nas x and y arguments is discouraged, and should instead be replaced\nwith using the separate wl_surface.offset request.\n\nWhen the bound wl_surface version is 5 or higher, passing any\nnon-zero x or y is a protocol violation, and will result in an\n'invalid_offset' error being raised. The x and y arguments are ignored\nand do not change the pending state. To achieve equivalent semantics,\nuse wl_surface.offset.\n\nSurface contents are double-buffered state, see wl_surface.commit.\n\nThe initial surface contents are void; there is no content.\nwl_surface.attach assigns the given wl_buffer as the pending\nwl_buffer. wl_surface.commit makes the pending wl_buffer the new\nsurface contents, and the size of the surface becomes the size\ncalculated from the wl_buffer, as described above. After commit,\nthere is no pending buffer until the next attach.\n\nCommitting a pending wl_buffer allows the compositor to read the\npixels in the wl_buffer. The compositor may access the pixels at\nany time after the wl_surface.commit request. When the compositor\nwill not access the pixels anymore, it will send the\nwl_buffer.release event. Only after receiving wl_buffer.release,\nthe client may reuse the wl_buffer. A wl_buffer that has been\nattached and then replaced by another attach instead of committed\nwill not receive a release event, and is not used by the\ncompositor.\n\nIf a pending wl_buffer has been committed to more than one wl_surface,\nthe delivery of wl_buffer.release events becomes undefined. A well\nbehaved client should not rely on wl_buffer.release events in this\ncase. Alternatively, a client could create multiple wl_buffer objects\nfrom the same backing storage or use wp_linux_buffer_release.\n\nDestroying the wl_buffer after wl_buffer.release does not change\nthe surface contents. Destroying the wl_buffer before wl_buffer.release\nis allowed as long as the underlying buffer storage isn't re-used (this\ncan happen e.g. on client process termination). However, if the client\ndestroys the wl_buffer before receiving the wl_buffer.release event and\nmutates the underlying buffer storage, the surface contents become\nundefined immediately.\n\nIf wl_surface.attach is sent with a NULL wl_buffer, the\nfollowing wl_surface.commit will remove the surface content.\n\nIf a pending wl_buffer has been destroyed, the result is not specified.\nMany compositors are known to remove the surface content on the following\nwl_surface.commit, but this behaviour is not universal. Clients seeking to\nmaximise compatibility should not destroy pending buffers and should\nensure that they explicitly remove content from surfaces, even after\ndestroying buffers.\n"] pub fn attach (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , buffer : Option < waybackend :: types :: ObjectId > , x : i32 , y : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , ATTACH) ? ; wire_msg_builder . add_object (wayland_fd , buffer) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DAMAGE : u16 = 2u16 ; # [doc = "mark part of the surface damaged\n\nThis request is used to describe the regions where the pending\nbuffer is different from the current surface contents, and where\nthe surface therefore needs to be repainted. The compositor\nignores the parts of the damage that fall outside of the surface.\n\nDamage is double-buffered state, see wl_surface.commit.\n\nThe damage rectangle is specified in surface-local coordinates,\nwhere x and y specify the upper left corner of the damage rectangle.\n\nThe initial value for pending damage is empty: no damage.\nwl_surface.damage adds pending damage: the new pending damage\nis the union of old pending damage and the given rectangle.\n\nwl_surface.commit assigns pending damage as the current damage,\nand clears pending damage. The server will clear the current\ndamage as it repaints the surface.\n\nNote! New clients should not use this request. Instead damage can be\nposted with wl_surface.damage_buffer which uses buffer coordinates\ninstead of surface coordinates.\n"] pub fn damage (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32 , width : i32 , height : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DAMAGE) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } pub const FRAME : u16 = 3u16 ; # [doc = "request a frame throttling hint\n\nRequest a notification when it is a good time to start drawing a new\nframe, by creating a frame callback. This is useful for throttling\nredrawing operations, and driving animations.\n\nWhen a client is animating on a wl_surface, it can use the 'frame'\nrequest to get notified when it is a good time to draw and commit the\nnext frame of animation. If the client commits an update earlier than\nthat, it is likely that some updates will not make it to the display,\nand the client is wasting resources by drawing too often.\n\nThe frame request will take effect on the next wl_surface.commit.\nThe notification will only be posted for one frame unless\nrequested again. For a wl_surface, the notifications are posted in\nthe order the frame requests were committed.\n\nThe server must send the notifications so that a client\nwill not send excessive updates, while still allowing\nthe highest possible update rate for clients that wait for the reply\nbefore drawing again. The server should give some time for the client\nto draw and commit after sending the frame callback events to let it\nhit the next output refresh.\n\nA server should avoid signaling the frame callbacks if the\nsurface is not visible in any way, e.g. the surface is off-screen,\nor completely obscured by other opaque surfaces.\n\nThe object returned by this request will be destroyed by the\ncompositor after the callback is fired and as such the client must not\nattempt to use it after that point.\n\nThe callback_data passed in the callback is the current time, in\nmilliseconds, with an undefined base.\n"] pub fn frame (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , callback : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , FRAME) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , callback) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_OPAQUE_REGION : u16 = 4u16 ; # [doc = "set opaque region\n\nThis request sets the region of the surface that contains\nopaque content.\n\nThe opaque region is an optimization hint for the compositor\nthat lets it optimize the redrawing of content behind opaque\nregions.  Setting an opaque region is not required for correct\nbehaviour, but marking transparent content as opaque will result\nin repaint artifacts.\n\nThe opaque region is specified in surface-local coordinates.\n\nThe compositor ignores the parts of the opaque region that fall\noutside of the surface.\n\nOpaque region is double-buffered state, see wl_surface.commit.\n\nwl_surface.set_opaque_region changes the pending opaque region.\nwl_surface.commit copies the pending region to the current region.\nOtherwise, the pending and current regions are never changed.\n\nThe initial value for an opaque region is empty. Setting the pending\nopaque region has copy semantics, and the wl_region object can be\ndestroyed immediately. A NULL wl_region causes the pending opaque\nregion to be set to empty.\n"] pub fn set_opaque_region (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , region : Option < waybackend :: types :: ObjectId >) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_OPAQUE_REGION) ? ; wire_msg_builder . add_object (wayland_fd , region) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_INPUT_REGION : u16 = 5u16 ; # [doc = "set input region\n\nThis request sets the region of the surface that can receive\npointer and touch events.\n\nInput events happening outside of this region will try the next\nsurface in the server surface stack. The compositor ignores the\nparts of the input region that fall outside of the surface.\n\nThe input region is specified in surface-local coordinates.\n\nInput region is double-buffered state, see wl_surface.commit.\n\nwl_surface.set_input_region changes the pending input region.\nwl_surface.commit copies the pending region to the current region.\nOtherwise the pending and current regions are never changed,\nexcept cursor and icon surfaces are special cases, see\nwl_pointer.set_cursor and wl_data_device.start_drag.\n\nThe initial value for an input region is infinite. That means the\nwhole surface will accept input. Setting the pending input region\nhas copy semantics, and the wl_region object can be destroyed\nimmediately. A NULL wl_region causes the input region to be set\nto infinite.\n"] pub fn set_input_region (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , region : Option < waybackend :: types :: ObjectId >) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_INPUT_REGION) ? ; wire_msg_builder . add_object (wayland_fd , region) ? ; wire_msg_builder . finish () ; Ok (()) } pub const COMMIT : u16 = 6u16 ; # [doc = "commit pending surface state\n\nSurface state (input, opaque, and damage regions, attached buffers,\netc.) is double-buffered. Protocol requests modify the pending state,\nas opposed to the active state in use by the compositor.\n\nA commit request atomically creates a content update from the pending\nstate, even if the pending state has not been touched. The content\nupdate is placed in a queue until it becomes active. After commit, the\nnew pending state is as documented for each related request.\n\nWhen the content update is applied, the wl_buffer is applied before all\nother state. This means that all coordinates in double-buffered state\nare relative to the newly attached wl_buffers, except for\nwl_surface.attach itself. If there is no newly attached wl_buffer, the\ncoordinates are relative to the previous content update.\n\nAll requests that need a commit to become effective are documented\nto affect double-buffered state.\n\nOther interfaces may add further double-buffered surface state.\n"] pub fn commit (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , COMMIT) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_BUFFER_TRANSFORM : u16 = 7u16 ; # [doc = "sets the buffer transformation\n\nThis request sets the transformation that the client has already applied\nto the content of the buffer. The accepted values for the transform\nparameter are the values for wl_output.transform.\n\nThe compositor applies the inverse of this transformation whenever it\nuses the buffer contents.\n\nBuffer transform is double-buffered state, see wl_surface.commit.\n\nA newly created surface has its buffer transformation set to normal.\n\nwl_surface.set_buffer_transform changes the pending buffer\ntransformation. wl_surface.commit copies the pending buffer\ntransformation to the current one. Otherwise, the pending and current\nvalues are never changed.\n\nThe purpose of this request is to allow clients to render content\naccording to the output transform, thus permitting the compositor to\nuse certain optimizations even if the display is rotated. Using\nhardware overlays and scanning out a client buffer for fullscreen\nsurfaces are examples of such optimizations. Those optimizations are\nhighly dependent on the compositor implementation, so the use of this\nrequest should be considered on a case-by-case basis.\n\nNote that if the transform value includes 90 or 270 degree rotation,\nthe width of the buffer will become the surface height and the height\nof the buffer will become the surface width.\n\nIf transform is not one of the values from the\nwl_output.transform enum the invalid_transform protocol error\nis raised.\n"] pub fn set_buffer_transform (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , transform : wl_output :: Transform) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_BUFFER_TRANSFORM) ? ; wire_msg_builder . add_u32 (wayland_fd , transform . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_BUFFER_SCALE : u16 = 8u16 ; # [doc = "sets the buffer scaling factor\n\nThis request sets an optional scaling factor on how the compositor\ninterprets the contents of the buffer attached to the window.\n\nBuffer scale is double-buffered state, see wl_surface.commit.\n\nA newly created surface has its buffer scale set to 1.\n\nwl_surface.set_buffer_scale changes the pending buffer scale.\nwl_surface.commit copies the pending buffer scale to the current one.\nOtherwise, the pending and current values are never changed.\n\nThe purpose of this request is to allow clients to supply higher\nresolution buffer data for use on high resolution outputs. It is\nintended that you pick the same buffer scale as the scale of the\noutput that the surface is displayed on. This means the compositor\ncan avoid scaling when rendering the surface on that output.\n\nNote that if the scale is larger than 1, then you have to attach\na buffer that is larger (by a factor of scale in each dimension)\nthan the desired surface size.\n\nIf scale is not greater than 0 the invalid_scale protocol error is\nraised.\n"] pub fn set_buffer_scale (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , scale : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_BUFFER_SCALE) ? ; wire_msg_builder . add_i32 (wayland_fd , scale) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DAMAGE_BUFFER : u16 = 9u16 ; # [doc = "mark part of the surface damaged using buffer coordinates\n\nThis request is used to describe the regions where the pending\nbuffer is different from the current surface contents, and where\nthe surface therefore needs to be repainted. The compositor\nignores the parts of the damage that fall outside of the surface.\n\nDamage is double-buffered state, see wl_surface.commit.\n\nThe damage rectangle is specified in buffer coordinates,\nwhere x and y specify the upper left corner of the damage rectangle.\n\nThe initial value for pending damage is empty: no damage.\nwl_surface.damage_buffer adds pending damage: the new pending\ndamage is the union of old pending damage and the given rectangle.\n\nwl_surface.commit assigns pending damage as the current damage,\nand clears pending damage. The server will clear the current\ndamage as it repaints the surface.\n\nThis request differs from wl_surface.damage in only one way - it\ntakes damage in buffer coordinates instead of surface-local\ncoordinates. While this generally is more intuitive than surface\ncoordinates, it is especially desirable when using wp_viewport\nor when a drawing library (like EGL) is unaware of buffer scale\nand buffer transform.\n\nNote: Because buffer transformation changes and damage requests may\nbe interleaved in the protocol stream, it is impossible to determine\nthe actual mapping between surface and buffer damage until\nwl_surface.commit time. Therefore, compositors wishing to take both\nkinds of damage into account will have to accumulate damage from the\ntwo requests separately and only transform from one to the other\nafter receiving the wl_surface.commit.\n"] pub fn damage_buffer (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32 , width : i32 , height : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DAMAGE_BUFFER) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } pub const OFFSET : u16 = 10u16 ; # [doc = "set the surface contents offset\n\nThe x and y arguments specify the location of the new pending\nbuffer's upper left corner, relative to the current buffer's upper\nleft corner, in surface-local coordinates. In other words, the\nx and y, combined with the new surface size define in which\ndirections the surface's size changes.\n\nSurface location offset is double-buffered state, see\nwl_surface.commit.\n\nThis request is semantically equivalent to and the replaces the x and y\narguments in the wl_surface.attach request in wl_surface versions prior\nto 5. See wl_surface.attach for details.\n"] pub fn offset (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , OFFSET) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "wl_surface error values\n\nThese errors can be emitted in response to wl_surface requests.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "buffer scale value is invalid"] invalid_scale = 0u32 , # [doc = "buffer transform value is invalid"] invalid_transform = 1u32 , # [doc = "buffer size is invalid"] invalid_size = 2u32 , # [doc = "buffer offset is invalid"] invalid_offset = 3u32 , # [doc = "surface was destroyed before its role object"] defunct_role_object = 4u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_scale : u32 = Error :: invalid_scale as u32 ; const invalid_transform : u32 = Error :: invalid_transform as u32 ; const invalid_size : u32 = Error :: invalid_size as u32 ; const invalid_offset : u32 = Error :: invalid_offset as u32 ; const defunct_role_object : u32 = Error :: defunct_role_object as u32 ; match value { invalid_scale => Ok (Self :: invalid_scale) , invalid_transform => Ok (Self :: invalid_transform) , invalid_size => Ok (Self :: invalid_size) , invalid_offset => Ok (Self :: invalid_offset) , defunct_role_object => Ok (Self :: defunct_role_object) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "group of input devices\n\nA seat is a group of keyboards, pointer and touch devices. This\nobject is published as a global during start up, or when such a\ndevice is hot plugged.  A seat typically has a pointer and\nmaintains a keyboard focus and a pointer focus.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_seat { use super :: * ; pub const MAX_VERSION : u32 = 9u32 ; pub const NAME : & str = "wl_seat" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "seat capabilities changed\n\nThis is emitted whenever a seat gains or loses the pointer,\nkeyboard or touch capabilities.  The argument is a capability\nenum containing the complete set of capabilities this seat has.\n\nWhen the pointer capability is added, a client may create a\nwl_pointer object using the wl_seat.get_pointer request. This object\nwill receive pointer events until the capability is removed in the\nfuture.\n\nWhen the pointer capability is removed, a client should destroy the\nwl_pointer objects associated with the seat where the capability was\nremoved, using the wl_pointer.release request. No further pointer\nevents will be received on these objects.\n\nIn some compositors, if a seat regains the pointer capability and a\nclient has a previously obtained wl_pointer object of version 4 or\nless, that object may start sending pointer events again. This\nbehavior is considered a misinterpretation of the intended behavior\nand must not be relied upon by the client. wl_pointer objects of\nversion 5 or later must not send events if created before the most\nrecent event notifying the client of an added pointer capability.\n\nThe above behavior also applies to wl_keyboard and wl_touch with the\nkeyboard and touch capabilities, respectively.\n"] fn capabilities (& mut self , sender_id : waybackend :: types :: ObjectId , capabilities : Capability) ; # [doc = "unique identifier for this seat\n\nIn a multi-seat configuration the seat name can be used by clients to\nhelp identify which physical devices the seat represents.\n\nThe seat name is a UTF-8 string with no convention defined for its\ncontents. Each name is unique among all wl_seat globals. The name is\nonly guaranteed to be unique for the current compositor instance.\n\nThe same seat names are used for all clients. Thus, the name can be\nshared across processes to refer to a specific wl_seat global.\n\nThe name event is sent after binding to the seat global. This event is\nonly sent once per seat object, and the name does not change over the\nlifetime of the wl_seat global.\n\nCompositors may re-use the same seat name if the wl_seat global is\ndestroyed and re-created later.\n"] fn name (& mut self , sender_id : waybackend :: types :: ObjectId , name : & str) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let capabilities : Capability = wire_msg . next_u32 () . try_into () ? ; state . capabilities (wire_msg . sender_id () , capabilities) ; Ok (()) } , 1u16 => { let name = wire_msg . next_string () ? ; state . name (wire_msg . sender_id () , name) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_seat" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const GET_POINTER : u16 = 0u16 ; # [doc = "return pointer object\n\nThe ID provided will be initialized to the wl_pointer interface\nfor this seat.\n\nThis request only takes effect if the seat has the pointer\ncapability, or has had the pointer capability in the past.\nIt is a protocol violation to issue this request on a seat that has\nnever had the pointer capability. The missing_capability error will\nbe sent in this case.\n"] pub fn get_pointer (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_POINTER) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_KEYBOARD : u16 = 1u16 ; # [doc = "return keyboard object\n\nThe ID provided will be initialized to the wl_keyboard interface\nfor this seat.\n\nThis request only takes effect if the seat has the keyboard\ncapability, or has had the keyboard capability in the past.\nIt is a protocol violation to issue this request on a seat that has\nnever had the keyboard capability. The missing_capability error will\nbe sent in this case.\n"] pub fn get_keyboard (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_KEYBOARD) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_TOUCH : u16 = 2u16 ; # [doc = "return touch object\n\nThe ID provided will be initialized to the wl_touch interface\nfor this seat.\n\nThis request only takes effect if the seat has the touch\ncapability, or has had the touch capability in the past.\nIt is a protocol violation to issue this request on a seat that has\nnever had the touch capability. The missing_capability error will\nbe sent in this case.\n"] pub fn get_touch (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_TOUCH) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RELEASE : u16 = 3u16 ; # [doc = "release the seat object\n\nUsing this request a client can tell the server that it is not going to\nuse the seat object anymore.\n\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = "seat capability bitmask\n\nThis is a bitmask of capabilities this seat has; if a member is\nset, then it is present on the seat.\n"] pub struct Capability : u32 { # [doc = "the seat has pointer devices"] const POINTER = 1u32 ; # [doc = "the seat has one or more keyboards"] const KEYBOARD = 2u32 ; # [doc = "the seat has touch devices"] const TOUCH = 4u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for Capability { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < Capability > for u32 { fn from (value : Capability) -> u32 { value . bits () } } # [doc = "wl_seat error values\n\nThese errors can be emitted in response to wl_seat requests.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "get_pointer, get_keyboard or get_touch called on seat without the matching capability"] missing_capability = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const missing_capability : u32 = Error :: missing_capability as u32 ; match value { missing_capability => Ok (Self :: missing_capability) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "pointer input device\n\nThe wl_pointer interface represents one or more input devices,\nsuch as mice, which control the pointer location and pointer_focus\nof a seat.\n\nThe wl_pointer interface generates motion, enter and leave\nevents for the surfaces that the pointer is located over,\nand button and axis events for button presses, button releases\nand scrolling.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_pointer { use super :: * ; pub const MAX_VERSION : u32 = 9u32 ; pub const NAME : & str = "wl_pointer" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "enter event\n\nNotification that this seat's pointer is focused on a certain\nsurface.\n\nWhen a seat's focus enters a surface, the pointer image\nis undefined and a client should respond to this event by setting\nan appropriate pointer image with the set_cursor request.\n"] fn enter (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : waybackend :: types :: ObjectId , surface_x : waybackend :: types :: WlFixed , surface_y : waybackend :: types :: WlFixed) ; # [doc = "leave event\n\nNotification that this seat's pointer is no longer focused on\na certain surface.\n\nThe leave notification is sent before the enter notification\nfor the new focus.\n"] fn leave (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : waybackend :: types :: ObjectId) ; # [doc = "pointer motion event\n\nNotification of pointer location change. The arguments\nsurface_x and surface_y are the location relative to the\nfocused surface.\n"] fn motion (& mut self , sender_id : waybackend :: types :: ObjectId , time : u32 , surface_x : waybackend :: types :: WlFixed , surface_y : waybackend :: types :: WlFixed) ; # [doc = "pointer button event\n\nMouse button click and release notifications.\n\nThe location of the click is given by the last motion or\nenter event.\nThe time argument is a timestamp with millisecond\ngranularity, with an undefined base.\n\nThe button is a button code as defined in the Linux kernel's\nlinux/input-event-codes.h header file, e.g. BTN_LEFT.\n\nAny 16-bit button code value is reserved for future additions to the\nkernel's event code list. All other button codes above 0xFFFF are\ncurrently undefined but may be used in future versions of this\nprotocol.\n"] fn button (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , time : u32 , button : u32 , _state : ButtonState) ; # [doc = "axis event\n\nScroll and other axis notifications.\n\nFor scroll events (vertical and horizontal scroll axes), the\nvalue parameter is the length of a vector along the specified\naxis in a coordinate space identical to those of motion events,\nrepresenting a relative movement along the specified axis.\n\nFor devices that support movements non-parallel to axes multiple\naxis events will be emitted.\n\nWhen applicable, for example for touch pads, the server can\nchoose to emit scroll events where the motion vector is\nequivalent to a motion event vector.\n\nWhen applicable, a client can transform its content relative to the\nscroll distance.\n"] fn axis (& mut self , sender_id : waybackend :: types :: ObjectId , time : u32 , axis : Axis , value : waybackend :: types :: WlFixed) ; # [doc = "end of a pointer event sequence\n\nIndicates the end of a set of events that logically belong together.\nA client is expected to accumulate the data in all events within the\nframe before proceeding.\n\nAll wl_pointer events before a wl_pointer.frame event belong\nlogically together. For example, in a diagonal scroll motion the\ncompositor will send an optional wl_pointer.axis_source event, two\nwl_pointer.axis events (horizontal and vertical) and finally a\nwl_pointer.frame event. The client may use this information to\ncalculate a diagonal vector for scrolling.\n\nWhen multiple wl_pointer.axis events occur within the same frame,\nthe motion vector is the combined motion of all events.\nWhen a wl_pointer.axis and a wl_pointer.axis_stop event occur within\nthe same frame, this indicates that axis movement in one axis has\nstopped but continues in the other axis.\nWhen multiple wl_pointer.axis_stop events occur within the same\nframe, this indicates that these axes stopped in the same instance.\n\nA wl_pointer.frame event is sent for every logical event group,\neven if the group only contains a single wl_pointer event.\nSpecifically, a client may get a sequence: motion, frame, button,\nframe, axis, frame, axis_stop, frame.\n\nThe wl_pointer.enter and wl_pointer.leave events are logical events\ngenerated by the compositor and not the hardware. These events are\nalso grouped by a wl_pointer.frame. When a pointer moves from one\nsurface to another, a compositor should group the\nwl_pointer.leave event within the same wl_pointer.frame.\nHowever, a client must not rely on wl_pointer.leave and\nwl_pointer.enter being in the same wl_pointer.frame.\nCompositor-specific policies may require the wl_pointer.leave and\nwl_pointer.enter event being split across multiple wl_pointer.frame\ngroups.\n"] fn frame (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "axis source event\n\nSource information for scroll and other axes.\n\nThis event does not occur on its own. It is sent before a\nwl_pointer.frame event and carries the source information for\nall events within that frame.\n\nThe source specifies how this event was generated. If the source is\nwl_pointer.axis_source.finger, a wl_pointer.axis_stop event will be\nsent when the user lifts the finger off the device.\n\nIf the source is wl_pointer.axis_source.wheel,\nwl_pointer.axis_source.wheel_tilt or\nwl_pointer.axis_source.continuous, a wl_pointer.axis_stop event may\nor may not be sent. Whether a compositor sends an axis_stop event\nfor these sources is hardware-specific and implementation-dependent;\nclients must not rely on receiving an axis_stop event for these\nscroll sources and should treat scroll sequences from these scroll\nsources as unterminated by default.\n\nThis event is optional. If the source is unknown for a particular\naxis event sequence, no event is sent.\nOnly one wl_pointer.axis_source event is permitted per frame.\n\nThe order of wl_pointer.axis_discrete and wl_pointer.axis_source is\nnot guaranteed.\n"] fn axis_source (& mut self , sender_id : waybackend :: types :: ObjectId , axis_source : AxisSource) ; # [doc = "axis stop event\n\nStop notification for scroll and other axes.\n\nFor some wl_pointer.axis_source types, a wl_pointer.axis_stop event\nis sent to notify a client that the axis sequence has terminated.\nThis enables the client to implement kinetic scrolling.\nSee the wl_pointer.axis_source documentation for information on when\nthis event may be generated.\n\nAny wl_pointer.axis events with the same axis_source after this\nevent should be considered as the start of a new axis motion.\n\nThe timestamp is to be interpreted identical to the timestamp in the\nwl_pointer.axis event. The timestamp value may be the same as a\npreceding wl_pointer.axis event.\n"] fn axis_stop (& mut self , sender_id : waybackend :: types :: ObjectId , time : u32 , axis : Axis) ; # [doc = "axis click event\n\nDiscrete step information for scroll and other axes.\n\nThis event carries the axis value of the wl_pointer.axis event in\ndiscrete steps (e.g. mouse wheel clicks).\n\nThis event is deprecated with wl_pointer version 8 - this event is not\nsent to clients supporting version 8 or later.\n\nThis event does not occur on its own, it is coupled with a\nwl_pointer.axis event that represents this axis value on a\ncontinuous scale. The protocol guarantees that each axis_discrete\nevent is always followed by exactly one axis event with the same\naxis number within the same wl_pointer.frame. Note that the protocol\nallows for other events to occur between the axis_discrete and\nits coupled axis event, including other axis_discrete or axis\nevents. A wl_pointer.frame must not contain more than one axis_discrete\nevent per axis type.\n\nThis event is optional; continuous scrolling devices\nlike two-finger scrolling on touchpads do not have discrete\nsteps and do not generate this event.\n\nThe discrete value carries the directional information. e.g. a value\nof -2 is two steps towards the negative direction of this axis.\n\nThe axis number is identical to the axis number in the associated\naxis event.\n\nThe order of wl_pointer.axis_discrete and wl_pointer.axis_source is\nnot guaranteed.\n\nDeprecated since interface version 8"] fn axis_discrete (& mut self , sender_id : waybackend :: types :: ObjectId , axis : Axis , discrete : i32) ; # [doc = "axis high-resolution scroll event\n\nDiscrete high-resolution scroll information.\n\nThis event carries high-resolution wheel scroll information,\nwith each multiple of 120 representing one logical scroll step\n(a wheel detent). For example, an axis_value120 of 30 is one quarter of\na logical scroll step in the positive direction, a value120 of\n-240 are two logical scroll steps in the negative direction within the\nsame hardware event.\nClients that rely on discrete scrolling should accumulate the\nvalue120 to multiples of 120 before processing the event.\n\nThe value120 must not be zero.\n\nThis event replaces the wl_pointer.axis_discrete event in clients\nsupporting wl_pointer version 8 or later.\n\nWhere a wl_pointer.axis_source event occurs in the same\nwl_pointer.frame, the axis source applies to this event.\n\nThe order of wl_pointer.axis_value120 and wl_pointer.axis_source is\nnot guaranteed.\n"] fn axis_value120 (& mut self , sender_id : waybackend :: types :: ObjectId , axis : Axis , value120 : i32) ; # [doc = "axis relative physical direction event\n\nRelative directional information of the entity causing the axis\nmotion.\n\nFor a wl_pointer.axis event, the wl_pointer.axis_relative_direction\nevent specifies the movement direction of the entity causing the\nwl_pointer.axis event. For example:\n- if a user's fingers on a touchpad move down and this\ncauses a wl_pointer.axis vertical_scroll down event, the physical\ndirection is 'identical'\n- if a user's fingers on a touchpad move down and this causes a\nwl_pointer.axis vertical_scroll up scroll up event ('natural\nscrolling'), the physical direction is 'inverted'.\n\nA client may use this information to adjust scroll motion of\ncomponents. Specifically, enabling natural scrolling causes the\ncontent to change direction compared to traditional scrolling.\nSome widgets like volume control sliders should usually match the\nphysical direction regardless of whether natural scrolling is\nactive. This event enables clients to match the scroll direction of\na widget to the physical direction.\n\nThis event does not occur on its own, it is coupled with a\nwl_pointer.axis event that represents this axis value.\nThe protocol guarantees that each axis_relative_direction event is\nalways followed by exactly one axis event with the same\naxis number within the same wl_pointer.frame. Note that the protocol\nallows for other events to occur between the axis_relative_direction\nand its coupled axis event.\n\nThe axis number is identical to the axis number in the associated\naxis event.\n\nThe order of wl_pointer.axis_relative_direction,\nwl_pointer.axis_discrete and wl_pointer.axis_source is not\nguaranteed.\n"] fn axis_relative_direction (& mut self , sender_id : waybackend :: types :: ObjectId , axis : Axis , direction : AxisRelativeDirection) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let serial = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; let surface_x = wire_msg . next_fixed () ; let surface_y = wire_msg . next_fixed () ; state . enter (wire_msg . sender_id () , serial , surface , surface_x , surface_y) ; Ok (()) } , 1u16 => { let serial = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; state . leave (wire_msg . sender_id () , serial , surface) ; Ok (()) } , 2u16 => { let time = wire_msg . next_u32 () ; let surface_x = wire_msg . next_fixed () ; let surface_y = wire_msg . next_fixed () ; state . motion (wire_msg . sender_id () , time , surface_x , surface_y) ; Ok (()) } , 3u16 => { let serial = wire_msg . next_u32 () ; let time = wire_msg . next_u32 () ; let button = wire_msg . next_u32 () ; let _state : ButtonState = wire_msg . next_u32 () . try_into () ? ; state . button (wire_msg . sender_id () , serial , time , button , _state) ; Ok (()) } , 4u16 => { let time = wire_msg . next_u32 () ; let axis : Axis = wire_msg . next_u32 () . try_into () ? ; let value = wire_msg . next_fixed () ; state . axis (wire_msg . sender_id () , time , axis , value) ; Ok (()) } , 5u16 => { state . frame (wire_msg . sender_id () ,) ; Ok (()) } , 6u16 => { let axis_source : AxisSource = wire_msg . next_u32 () . try_into () ? ; state . axis_source (wire_msg . sender_id () , axis_source) ; Ok (()) } , 7u16 => { let time = wire_msg . next_u32 () ; let axis : Axis = wire_msg . next_u32 () . try_into () ? ; state . axis_stop (wire_msg . sender_id () , time , axis) ; Ok (()) } , 8u16 => { let axis : Axis = wire_msg . next_u32 () . try_into () ? ; let discrete = wire_msg . next_i32 () ; state . axis_discrete (wire_msg . sender_id () , axis , discrete) ; Ok (()) } , 9u16 => { let axis : Axis = wire_msg . next_u32 () . try_into () ? ; let value120 = wire_msg . next_i32 () ; state . axis_value120 (wire_msg . sender_id () , axis , value120) ; Ok (()) } , 10u16 => { let axis : Axis = wire_msg . next_u32 () . try_into () ? ; let direction : AxisRelativeDirection = wire_msg . next_u32 () . try_into () ? ; state . axis_relative_direction (wire_msg . sender_id () , axis , direction) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_pointer" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const SET_CURSOR : u16 = 0u16 ; # [doc = "set the pointer surface\n\nSet the pointer surface, i.e., the surface that contains the\npointer image (cursor). This request gives the surface the role\nof a cursor. If the surface already has another role, it raises\na protocol error.\n\nThe cursor actually changes only if the pointer\nfocus for this device is one of the requesting client's surfaces\nor the surface parameter is the current pointer surface. If\nthere was a previous surface set with this request it is\nreplaced. If surface is NULL, the pointer image is hidden.\n\nThe parameters hotspot_x and hotspot_y define the position of\nthe pointer surface relative to the pointer location. Its\ntop-left corner is always at (x, y) - (hotspot_x, hotspot_y),\nwhere (x, y) are the coordinates of the pointer location, in\nsurface-local coordinates.\n\nOn wl_surface.offset requests to the pointer surface, hotspot_x\nand hotspot_y are decremented by the x and y parameters\npassed to the request. The offset must be applied by\nwl_surface.commit as usual.\n\nThe hotspot can also be updated by passing the currently set\npointer surface to this request with new values for hotspot_x\nand hotspot_y.\n\nThe input region is ignored for wl_surfaces with the role of\na cursor. When the use as a cursor ends, the wl_surface is\nunmapped.\n\nThe serial parameter must match the latest wl_pointer.enter\nserial number sent to the client. Otherwise the request will be\nignored.\n"] pub fn set_cursor (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : Option < waybackend :: types :: ObjectId > , hotspot_x : i32 , hotspot_y : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_CURSOR) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . add_object (wayland_fd , surface) ? ; wire_msg_builder . add_i32 (wayland_fd , hotspot_x) ? ; wire_msg_builder . add_i32 (wayland_fd , hotspot_y) ? ; wire_msg_builder . finish () ; Ok (()) } pub const RELEASE : u16 = 1u16 ; # [doc = "release the pointer object\n\nUsing this request a client can tell the server that it is not going to\nuse the pointer object anymore.\n\nThis request destroys the pointer proxy object, so clients must not call\nwl_pointer_destroy() after using this request.\n\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "given wl_surface has another role"] role = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const role : u32 = Error :: role as u32 ; match value { role => Ok (Self :: role) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } # [doc = "physical button state\n\nDescribes the physical state of a button that produced the button\nevent.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum ButtonState { # [doc = "the button is not pressed"] released = 0u32 , # [doc = "the button is pressed"] pressed = 1u32 , } impl std :: convert :: TryFrom < u32 > for ButtonState { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const released : u32 = ButtonState :: released as u32 ; const pressed : u32 = ButtonState :: pressed as u32 ; match value { released => Ok (Self :: released) , pressed => Ok (Self :: pressed) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("ButtonState" , otherwise))) , } } } impl std :: convert :: From < ButtonState > for u32 { fn from (value : ButtonState) -> u32 { value as u32 } } # [doc = "axis types\n\nDescribes the axis types of scroll events.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Axis { # [doc = "vertical axis"] vertical_scroll = 0u32 , # [doc = "horizontal axis"] horizontal_scroll = 1u32 , } impl std :: convert :: TryFrom < u32 > for Axis { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const vertical_scroll : u32 = Axis :: vertical_scroll as u32 ; const horizontal_scroll : u32 = Axis :: horizontal_scroll as u32 ; match value { vertical_scroll => Ok (Self :: vertical_scroll) , horizontal_scroll => Ok (Self :: horizontal_scroll) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Axis" , otherwise))) , } } } impl std :: convert :: From < Axis > for u32 { fn from (value : Axis) -> u32 { value as u32 } } # [doc = "axis source types\n\nDescribes the source types for axis events. This indicates to the\nclient how an axis event was physically generated; a client may\nadjust the user interface accordingly. For example, scroll events\nfrom a \"finger\" source may be in a smooth coordinate space with\nkinetic scrolling whereas a \"wheel\" source may be in discrete steps\nof a number of lines.\n\nThe \"continuous\" axis source is a device generating events in a\ncontinuous coordinate space, but using something other than a\nfinger. One example for this source is button-based scrolling where\nthe vertical motion of a device is converted to scroll events while\na button is held down.\n\nThe \"wheel tilt\" axis source indicates that the actual device is a\nwheel but the scroll event is not caused by a rotation but a\n(usually sideways) tilt of the wheel.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum AxisSource { # [doc = "a physical wheel rotation"] wheel = 0u32 , # [doc = "finger on a touch surface"] finger = 1u32 , # [doc = "continuous coordinate space"] continuous = 2u32 , # [doc = "a physical wheel tilt"] wheel_tilt = 3u32 , } impl std :: convert :: TryFrom < u32 > for AxisSource { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const wheel : u32 = AxisSource :: wheel as u32 ; const finger : u32 = AxisSource :: finger as u32 ; const continuous : u32 = AxisSource :: continuous as u32 ; const wheel_tilt : u32 = AxisSource :: wheel_tilt as u32 ; match value { wheel => Ok (Self :: wheel) , finger => Ok (Self :: finger) , continuous => Ok (Self :: continuous) , wheel_tilt => Ok (Self :: wheel_tilt) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("AxisSource" , otherwise))) , } } } impl std :: convert :: From < AxisSource > for u32 { fn from (value : AxisSource) -> u32 { value as u32 } } # [doc = "axis relative direction\n\nThis specifies the direction of the physical motion that caused a\nwl_pointer.axis event, relative to the wl_pointer.axis direction.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum AxisRelativeDirection { # [doc = "physical motion matches axis direction"] identical = 0u32 , # [doc = "physical motion is the inverse of the axis direction"] inverted = 1u32 , } impl std :: convert :: TryFrom < u32 > for AxisRelativeDirection { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const identical : u32 = AxisRelativeDirection :: identical as u32 ; const inverted : u32 = AxisRelativeDirection :: inverted as u32 ; match value { identical => Ok (Self :: identical) , inverted => Ok (Self :: inverted) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("AxisRelativeDirection" , otherwise))) , } } } impl std :: convert :: From < AxisRelativeDirection > for u32 { fn from (value : AxisRelativeDirection) -> u32 { value as u32 } } } # [doc = "keyboard input device\n\nThe wl_keyboard interface represents one or more keyboards\nassociated with a seat.\n\nEach wl_keyboard has the following logical state:\n\n- an active surface (possibly null),\n- the keys currently logically down,\n- the active modifiers,\n- the active group.\n\nBy default, the active surface is null, the keys currently logically down\nare empty, the active modifiers and the active group are 0.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_keyboard { use super :: * ; pub const MAX_VERSION : u32 = 9u32 ; pub const NAME : & str = "wl_keyboard" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "keyboard mapping\n\nThis event provides a file descriptor to the client which can be\nmemory-mapped in read-only mode to provide a keyboard mapping\ndescription.\n\nFrom version 7 onwards, the fd must be mapped with MAP_PRIVATE by\nthe recipient, as MAP_SHARED may fail.\n"] fn keymap (& mut self , sender_id : waybackend :: types :: ObjectId , format : KeymapFormat , fd : & impl waybackend :: rustix :: fd :: AsRawFd , size : u32) ; # [doc = "enter event\n\nNotification that this seat's keyboard focus is on a certain\nsurface.\n\nThe compositor must send the wl_keyboard.modifiers event after this\nevent.\n\nIn the wl_keyboard logical state, this event sets the active surface to\nthe surface argument and the keys currently logically down to the keys\nin the keys argument. The compositor must not send this event if the\nwl_keyboard already had an active surface immediately before this event.\n"] fn enter (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : waybackend :: types :: ObjectId , keys : & [u8]) ; # [doc = "leave event\n\nNotification that this seat's keyboard focus is no longer on\na certain surface.\n\nThe leave notification is sent before the enter notification\nfor the new focus.\n\nIn the wl_keyboard logical state, this event resets all values to their\ndefaults. The compositor must not send this event if the active surface\nof the wl_keyboard was not equal to the surface argument immediately\nbefore this event.\n"] fn leave (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , surface : waybackend :: types :: ObjectId) ; # [doc = "key event\n\nA key was pressed or released.\nThe time argument is a timestamp with millisecond\ngranularity, with an undefined base.\n\nThe key is a platform-specific key code that can be interpreted\nby feeding it to the keyboard mapping (see the keymap event).\n\nIf this event produces a change in modifiers, then the resulting\nwl_keyboard.modifiers event must be sent after this event.\n\nIn the wl_keyboard logical state, this event adds the key to the keys\ncurrently logically down (if the state argument is pressed) or removes\nthe key from the keys currently logically down (if the state argument is\nreleased). The compositor must not send this event if the wl_keyboard\ndid not have an active surface immediately before this event. The\ncompositor must not send this event if state is pressed (resp. released)\nand the key was already logically down (resp. was not logically down)\nimmediately before this event.\n"] fn key (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , time : u32 , key : u32 , _state : KeyState) ; # [doc = "modifier and group state\n\nNotifies clients that the modifier and/or group state has\nchanged, and it should update its local state.\n\nThe compositor may send this event without a surface of the client\nhaving keyboard focus, for example to tie modifier information to\npointer focus instead. If a modifier event with pressed modifiers is sent\nwithout a prior enter event, the client can assume the modifier state is\nvalid until it receives the next wl_keyboard.modifiers event. In order to\nreset the modifier state again, the compositor can send a\nwl_keyboard.modifiers event with no pressed modifiers.\n\nIn the wl_keyboard logical state, this event updates the modifiers and\ngroup.\n"] fn modifiers (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , mods_depressed : u32 , mods_latched : u32 , mods_locked : u32 , group : u32) ; # [doc = "repeat rate and delay\n\nInforms the client about the keyboard's repeat rate and delay.\n\nThis event is sent as soon as the wl_keyboard object has been created,\nand is guaranteed to be received by the client before any key press\nevent.\n\nNegative values for either rate or delay are illegal. A rate of zero\nwill disable any repeating (regardless of the value of delay).\n\nThis event can be sent later on as well with a new value if necessary,\nso clients should continue listening for the event past the creation\nof wl_keyboard.\n"] fn repeat_info (& mut self , sender_id : waybackend :: types :: ObjectId , rate : i32 , delay : i32) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let format : KeymapFormat = wire_msg . next_u32 () . try_into () ? ; let fd = wire_msg . next_fd () ; let fd = & fd ; let size = wire_msg . next_u32 () ; state . keymap (wire_msg . sender_id () , format , fd , size) ; Ok (()) } , 1u16 => { let serial = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; let keys = wire_msg . next_array () ; state . enter (wire_msg . sender_id () , serial , surface , keys) ; Ok (()) } , 2u16 => { let serial = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; state . leave (wire_msg . sender_id () , serial , surface) ; Ok (()) } , 3u16 => { let serial = wire_msg . next_u32 () ; let time = wire_msg . next_u32 () ; let key = wire_msg . next_u32 () ; let _state : KeyState = wire_msg . next_u32 () . try_into () ? ; state . key (wire_msg . sender_id () , serial , time , key , _state) ; Ok (()) } , 4u16 => { let serial = wire_msg . next_u32 () ; let mods_depressed = wire_msg . next_u32 () ; let mods_latched = wire_msg . next_u32 () ; let mods_locked = wire_msg . next_u32 () ; let group = wire_msg . next_u32 () ; state . modifiers (wire_msg . sender_id () , serial , mods_depressed , mods_latched , mods_locked , group) ; Ok (()) } , 5u16 => { let rate = wire_msg . next_i32 () ; let delay = wire_msg . next_i32 () ; state . repeat_info (wire_msg . sender_id () , rate , delay) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_keyboard" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const RELEASE : u16 = 0u16 ; # [doc = "release the keyboard object\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "keyboard mapping format\n\nThis specifies the format of the keymap provided to the\nclient with the wl_keyboard.keymap event.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum KeymapFormat { # [doc = "no keymap; client must understand how to interpret the raw keycode"] no_keymap = 0u32 , # [doc = "libxkbcommon compatible, null-terminated string; to determine the xkb keycode, clients must add 8 to the key event keycode"] xkb_v1 = 1u32 , } impl std :: convert :: TryFrom < u32 > for KeymapFormat { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const no_keymap : u32 = KeymapFormat :: no_keymap as u32 ; const xkb_v1 : u32 = KeymapFormat :: xkb_v1 as u32 ; match value { no_keymap => Ok (Self :: no_keymap) , xkb_v1 => Ok (Self :: xkb_v1) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("KeymapFormat" , otherwise))) , } } } impl std :: convert :: From < KeymapFormat > for u32 { fn from (value : KeymapFormat) -> u32 { value as u32 } } # [doc = "physical key state\n\nDescribes the physical state of a key that produced the key event.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum KeyState { # [doc = "key is not pressed"] released = 0u32 , # [doc = "key is pressed"] pressed = 1u32 , } impl std :: convert :: TryFrom < u32 > for KeyState { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const released : u32 = KeyState :: released as u32 ; const pressed : u32 = KeyState :: pressed as u32 ; match value { released => Ok (Self :: released) , pressed => Ok (Self :: pressed) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("KeyState" , otherwise))) , } } } impl std :: convert :: From < KeyState > for u32 { fn from (value : KeyState) -> u32 { value as u32 } } } # [doc = "touchscreen input device\n\nThe wl_touch interface represents a touchscreen\nassociated with a seat.\n\nTouch interactions can consist of one or more contacts.\nFor each contact, a series of events is generated, starting\nwith a down event, followed by zero or more motion events,\nand ending with an up event. Events relating to the same\ncontact point can be identified by the ID of the sequence.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_touch { use super :: * ; pub const MAX_VERSION : u32 = 9u32 ; pub const NAME : & str = "wl_touch" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "touch down event and beginning of a touch sequence\n\nA new touch point has appeared on the surface. This touch point is\nassigned a unique ID. Future events from this touch point reference\nthis ID. The ID ceases to be valid after a touch up event and may be\nreused in the future.\n"] fn down (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , time : u32 , surface : waybackend :: types :: ObjectId , id : i32 , x : waybackend :: types :: WlFixed , y : waybackend :: types :: WlFixed) ; # [doc = "end of a touch event sequence\n\nThe touch point has disappeared. No further events will be sent for\nthis touch point and the touch point's ID is released and may be\nreused in a future touch down event.\n"] fn up (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , time : u32 , id : i32) ; # [doc = "update of touch point coordinates\n\nA touch point has changed coordinates.\n"] fn motion (& mut self , sender_id : waybackend :: types :: ObjectId , time : u32 , id : i32 , x : waybackend :: types :: WlFixed , y : waybackend :: types :: WlFixed) ; # [doc = "end of touch frame event\n\nIndicates the end of a set of events that logically belong together.\nA client is expected to accumulate the data in all events within the\nframe before proceeding.\n\nA wl_touch.frame terminates at least one event but otherwise no\nguarantee is provided about the set of events within a frame. A client\nmust assume that any state not updated in a frame is unchanged from the\npreviously known state.\n"] fn frame (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "touch session cancelled\n\nSent if the compositor decides the touch stream is a global\ngesture. No further events are sent to the clients from that\nparticular gesture. Touch cancellation applies to all touch points\ncurrently active on this client's surface. The client is\nresponsible for finalizing the touch points, future touch points on\nthis surface may reuse the touch point ID.\n\nNo frame event is required after the cancel event.\n"] fn cancel (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "update shape of touch point\n\nSent when a touchpoint has changed its shape.\n\nThis event does not occur on its own. It is sent before a\nwl_touch.frame event and carries the new shape information for\nany previously reported, or new touch points of that frame.\n\nOther events describing the touch point such as wl_touch.down,\nwl_touch.motion or wl_touch.orientation may be sent within the\nsame wl_touch.frame. A client should treat these events as a single\nlogical touch point update. The order of wl_touch.shape,\nwl_touch.orientation and wl_touch.motion is not guaranteed.\nA wl_touch.down event is guaranteed to occur before the first\nwl_touch.shape event for this touch ID but both events may occur within\nthe same wl_touch.frame.\n\nA touchpoint shape is approximated by an ellipse through the major and\nminor axis length. The major axis length describes the longer diameter\nof the ellipse, while the minor axis length describes the shorter\ndiameter. Major and minor are orthogonal and both are specified in\nsurface-local coordinates. The center of the ellipse is always at the\ntouchpoint location as reported by wl_touch.down or wl_touch.move.\n\nThis event is only sent by the compositor if the touch device supports\nshape reports. The client has to make reasonable assumptions about the\nshape if it did not receive this event.\n"] fn shape (& mut self , sender_id : waybackend :: types :: ObjectId , id : i32 , major : waybackend :: types :: WlFixed , minor : waybackend :: types :: WlFixed) ; # [doc = "update orientation of touch point\n\nSent when a touchpoint has changed its orientation.\n\nThis event does not occur on its own. It is sent before a\nwl_touch.frame event and carries the new shape information for\nany previously reported, or new touch points of that frame.\n\nOther events describing the touch point such as wl_touch.down,\nwl_touch.motion or wl_touch.shape may be sent within the\nsame wl_touch.frame. A client should treat these events as a single\nlogical touch point update. The order of wl_touch.shape,\nwl_touch.orientation and wl_touch.motion is not guaranteed.\nA wl_touch.down event is guaranteed to occur before the first\nwl_touch.orientation event for this touch ID but both events may occur\nwithin the same wl_touch.frame.\n\nThe orientation describes the clockwise angle of a touchpoint's major\naxis to the positive surface y-axis and is normalized to the -180 to\n+180 degree range. The granularity of orientation depends on the touch\ndevice, some devices only support binary rotation values between 0 and\n90 degrees.\n\nThis event is only sent by the compositor if the touch device supports\norientation reports.\n"] fn orientation (& mut self , sender_id : waybackend :: types :: ObjectId , id : i32 , orientation : waybackend :: types :: WlFixed) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let serial = wire_msg . next_u32 () ; let time = wire_msg . next_u32 () ; let surface = wire_msg . next_object () . ok_or (waybackend :: wire :: Error :: NullObjectId) ? ; let id = wire_msg . next_i32 () ; let x = wire_msg . next_fixed () ; let y = wire_msg . next_fixed () ; state . down (wire_msg . sender_id () , serial , time , surface , id , x , y) ; Ok (()) } , 1u16 => { let serial = wire_msg . next_u32 () ; let time = wire_msg . next_u32 () ; let id = wire_msg . next_i32 () ; state . up (wire_msg . sender_id () , serial , time , id) ; Ok (()) } , 2u16 => { let time = wire_msg . next_u32 () ; let id = wire_msg . next_i32 () ; let x = wire_msg . next_fixed () ; let y = wire_msg . next_fixed () ; state . motion (wire_msg . sender_id () , time , id , x , y) ; Ok (()) } , 3u16 => { state . frame (wire_msg . sender_id () ,) ; Ok (()) } , 4u16 => { state . cancel (wire_msg . sender_id () ,) ; Ok (()) } , 5u16 => { let id = wire_msg . next_i32 () ; let major = wire_msg . next_fixed () ; let minor = wire_msg . next_fixed () ; state . shape (wire_msg . sender_id () , id , major , minor) ; Ok (()) } , 6u16 => { let id = wire_msg . next_i32 () ; let orientation = wire_msg . next_fixed () ; state . orientation (wire_msg . sender_id () , id , orientation) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_touch" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const RELEASE : u16 = 0u16 ; # [doc = "release the touch object\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "compositor output region\n\nAn output describes part of the compositor geometry.  The\ncompositor works in the 'compositor coordinate system' and an\noutput corresponds to a rectangular area in that space that is\nactually visible.  This typically corresponds to a monitor that\ndisplays part of the compositor space.  This object is published\nas global during start up, or when a monitor is hotplugged.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_output { use super :: * ; pub const MAX_VERSION : u32 = 4u32 ; pub const NAME : & str = "wl_output" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "properties of the output\n\nThe geometry event describes geometric properties of the output.\nThe event is sent when binding to the output object and whenever\nany of the properties change.\n\nThe physical size can be set to zero if it doesn't make sense for this\noutput (e.g. for projectors or virtual outputs).\n\nThe geometry event will be followed by a done event (starting from\nversion 2).\n\nClients should use wl_surface.preferred_buffer_transform instead of the\ntransform advertised by this event to find the preferred buffer\ntransform to use for a surface.\n\nNote: wl_output only advertises partial information about the output\nposition and identification. Some compositors, for instance those not\nimplementing a desktop-style output layout or those exposing virtual\noutputs, might fake this information. Instead of using x and y, clients\nshould use xdg_output.logical_position. Instead of using make and model,\nclients should use name and description.\n"] fn geometry (& mut self , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32 , physical_width : i32 , physical_height : i32 , subpixel : Subpixel , make : & str , model : & str , transform : Transform) ; # [doc = "advertise available modes for the output\n\nThe mode event describes an available mode for the output.\n\nThe event is sent when binding to the output object and there\nwill always be one mode, the current mode.  The event is sent\nagain if an output changes mode, for the mode that is now\ncurrent.  In other words, the current mode is always the last\nmode that was received with the current flag set.\n\nNon-current modes are deprecated. A compositor can decide to only\nadvertise the current mode and never send other modes. Clients\nshould not rely on non-current modes.\n\nThe size of a mode is given in physical hardware units of\nthe output device. This is not necessarily the same as\nthe output size in the global compositor space. For instance,\nthe output may be scaled, as described in wl_output.scale,\nor transformed, as described in wl_output.transform. Clients\nwilling to retrieve the output size in the global compositor\nspace should use xdg_output.logical_size instead.\n\nThe vertical refresh rate can be set to zero if it doesn't make\nsense for this output (e.g. for virtual outputs).\n\nThe mode event will be followed by a done event (starting from\nversion 2).\n\nClients should not use the refresh rate to schedule frames. Instead,\nthey should use the wl_surface.frame event or the presentation-time\nprotocol.\n\nNote: this information is not always meaningful for all outputs. Some\ncompositors, such as those exposing virtual outputs, might fake the\nrefresh rate or the size.\n"] fn mode (& mut self , sender_id : waybackend :: types :: ObjectId , flags : Mode , width : i32 , height : i32 , refresh : i32) ; # [doc = "sent all information about output\n\nThis event is sent after all other properties have been\nsent after binding to the output object and after any\nother property changes done after that. This allows\nchanges to the output properties to be seen as\natomic, even if they happen via multiple events.\n"] fn done (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; # [doc = "output scaling properties\n\nThis event contains scaling geometry information\nthat is not in the geometry event. It may be sent after\nbinding the output object or if the output scale changes\nlater. The compositor will emit a non-zero, positive\nvalue for scale. If it is not sent, the client should\nassume a scale of 1.\n\nA scale larger than 1 means that the compositor will\nautomatically scale surface buffers by this amount\nwhen rendering. This is used for very high resolution\ndisplays where applications rendering at the native\nresolution would be too small to be legible.\n\nClients should use wl_surface.preferred_buffer_scale\ninstead of this event to find the preferred buffer\nscale to use for a surface.\n\nThe scale event will be followed by a done event.\n"] fn scale (& mut self , sender_id : waybackend :: types :: ObjectId , factor : i32) ; # [doc = "name of this output\n\nMany compositors will assign user-friendly names to their outputs, show\nthem to the user, allow the user to refer to an output, etc. The client\nmay wish to know this name as well to offer the user similar behaviors.\n\nThe name is a UTF-8 string with no convention defined for its contents.\nEach name is unique among all wl_output globals. The name is only\nguaranteed to be unique for the compositor instance.\n\nThe same output name is used for all clients for a given wl_output\nglobal. Thus, the name can be shared across processes to refer to a\nspecific wl_output global.\n\nThe name is not guaranteed to be persistent across sessions, thus cannot\nbe used to reliably identify an output in e.g. configuration files.\n\nExamples of names include 'HDMI-A-1', 'WL-1', 'X11-1', etc. However, do\nnot assume that the name is a reflection of an underlying DRM connector,\nX11 connection, etc.\n\nThe name event is sent after binding the output object. This event is\nonly sent once per output object, and the name does not change over the\nlifetime of the wl_output global.\n\nCompositors may re-use the same output name if the wl_output global is\ndestroyed and re-created later. Compositors should avoid re-using the\nsame name if possible.\n\nThe name event will be followed by a done event.\n"] fn name (& mut self , sender_id : waybackend :: types :: ObjectId , name : & str) ; # [doc = "human-readable description of this output\n\nMany compositors can produce human-readable descriptions of their\noutputs. The client may wish to know this description as well, e.g. for\noutput selection purposes.\n\nThe description is a UTF-8 string with no convention defined for its\ncontents. The description is not guaranteed to be unique among all\nwl_output globals. Examples might include 'Foocorp 11\" Display' or\n'Virtual X11 output via :1'.\n\nThe description event is sent after binding the output object and\nwhenever the description changes. The description is optional, and may\nnot be sent at all.\n\nThe description event will be followed by a done event.\n"] fn description (& mut self , sender_id : waybackend :: types :: ObjectId , description : & str) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let x = wire_msg . next_i32 () ; let y = wire_msg . next_i32 () ; let physical_width = wire_msg . next_i32 () ; let physical_height = wire_msg . next_i32 () ; let subpixel : Subpixel = wire_msg . next_u32 () . try_into () ? ; let make = wire_msg . next_string () ? ; let model = wire_msg . next_string () ? ; let transform : Transform = wire_msg . next_u32 () . try_into () ? ; state . geometry (wire_msg . sender_id () , x , y , physical_width , physical_height , subpixel , make , model , transform) ; Ok (()) } , 1u16 => { let flags : Mode = wire_msg . next_u32 () . try_into () ? ; let width = wire_msg . next_i32 () ; let height = wire_msg . next_i32 () ; let refresh = wire_msg . next_i32 () ; state . mode (wire_msg . sender_id () , flags , width , height , refresh) ; Ok (()) } , 2u16 => { state . done (wire_msg . sender_id () ,) ; Ok (()) } , 3u16 => { let factor = wire_msg . next_i32 () ; state . scale (wire_msg . sender_id () , factor) ; Ok (()) } , 4u16 => { let name = wire_msg . next_string () ? ; state . name (wire_msg . sender_id () , name) ; Ok (()) } , 5u16 => { let description = wire_msg . next_string () ? ; state . description (wire_msg . sender_id () , description) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_output" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const RELEASE : u16 = 0u16 ; # [doc = "release the output object\n\nUsing this request a client can tell the server that it is not going to\nuse the output object anymore.\n\nTHIS IS A DESTRUCTOR"] pub fn release (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , RELEASE) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "subpixel geometry information\n\nThis enumeration describes how the physical\npixels on an output are laid out.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Subpixel { # [doc = "unknown geometry"] unknown = 0u32 , # [doc = "no geometry"] none = 1u32 , # [doc = "horizontal RGB"] horizontal_rgb = 2u32 , # [doc = "horizontal BGR"] horizontal_bgr = 3u32 , # [doc = "vertical RGB"] vertical_rgb = 4u32 , # [doc = "vertical BGR"] vertical_bgr = 5u32 , } impl std :: convert :: TryFrom < u32 > for Subpixel { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const unknown : u32 = Subpixel :: unknown as u32 ; const none : u32 = Subpixel :: none as u32 ; const horizontal_rgb : u32 = Subpixel :: horizontal_rgb as u32 ; const horizontal_bgr : u32 = Subpixel :: horizontal_bgr as u32 ; const vertical_rgb : u32 = Subpixel :: vertical_rgb as u32 ; const vertical_bgr : u32 = Subpixel :: vertical_bgr as u32 ; match value { unknown => Ok (Self :: unknown) , none => Ok (Self :: none) , horizontal_rgb => Ok (Self :: horizontal_rgb) , horizontal_bgr => Ok (Self :: horizontal_bgr) , vertical_rgb => Ok (Self :: vertical_rgb) , vertical_bgr => Ok (Self :: vertical_bgr) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Subpixel" , otherwise))) , } } } impl std :: convert :: From < Subpixel > for u32 { fn from (value : Subpixel) -> u32 { value as u32 } } # [doc = "transformation applied to buffer contents\n\nThis describes transformations that clients and compositors apply to\nbuffer contents.\n\nThe flipped values correspond to an initial flip around a\nvertical axis followed by rotation.\n\nThe purpose is mainly to allow clients to render accordingly and\ntell the compositor, so that for fullscreen surfaces, the\ncompositor will still be able to scan out directly from client\nsurfaces.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Transform { # [doc = "no transform"] normal = 0u32 , # [doc = "90 degrees counter-clockwise"] _90 = 1u32 , # [doc = "180 degrees counter-clockwise"] _180 = 2u32 , # [doc = "270 degrees counter-clockwise"] _270 = 3u32 , # [doc = "180 degree flip around a vertical axis"] flipped = 4u32 , # [doc = "flip and rotate 90 degrees counter-clockwise"] flipped_90 = 5u32 , # [doc = "flip and rotate 180 degrees counter-clockwise"] flipped_180 = 6u32 , # [doc = "flip and rotate 270 degrees counter-clockwise"] flipped_270 = 7u32 , } impl std :: convert :: TryFrom < u32 > for Transform { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const normal : u32 = Transform :: normal as u32 ; const _90 : u32 = Transform :: _90 as u32 ; const _180 : u32 = Transform :: _180 as u32 ; const _270 : u32 = Transform :: _270 as u32 ; const flipped : u32 = Transform :: flipped as u32 ; const flipped_90 : u32 = Transform :: flipped_90 as u32 ; const flipped_180 : u32 = Transform :: flipped_180 as u32 ; const flipped_270 : u32 = Transform :: flipped_270 as u32 ; match value { normal => Ok (Self :: normal) , _90 => Ok (Self :: _90) , _180 => Ok (Self :: _180) , _270 => Ok (Self :: _270) , flipped => Ok (Self :: flipped) , flipped_90 => Ok (Self :: flipped_90) , flipped_180 => Ok (Self :: flipped_180) , flipped_270 => Ok (Self :: flipped_270) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Transform" , otherwise))) , } } } impl std :: convert :: From < Transform > for u32 { fn from (value : Transform) -> u32 { value as u32 } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = "mode information\n\nThese flags describe properties of an output mode.\nThey are used in the flags bitfield of the mode event.\n"] pub struct Mode : u32 { # [doc = "indicates this is the current mode"] const CURRENT = 1u32 ; # [doc = "indicates this is the preferred mode"] const PREFERRED = 2u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for Mode { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < Mode > for u32 { fn from (value : Mode) -> u32 { value . bits () } } } # [doc = "region interface\n\nA region object describes an area.\n\nRegion objects are used to describe the opaque and input\nregions of a surface.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_region { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_region" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_region" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "destroy region\n\nDestroy the region.  This will invalidate the object ID.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const ADD : u16 = 1u16 ; # [doc = "add rectangle to region\n\nAdd the specified rectangle to the region.\n"] pub fn add (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32 , width : i32 , height : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , ADD) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SUBTRACT : u16 = 2u16 ; # [doc = "subtract rectangle from region\n\nSubtract the specified rectangle from the region.\n"] pub fn subtract (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32 , width : i32 , height : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SUBTRACT) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } } } # [doc = "sub-surface compositing\n\nThe global interface exposing sub-surface compositing capabilities.\nA wl_surface, that has sub-surfaces associated, is called the\nparent surface. Sub-surfaces can be arbitrarily nested and create\na tree of sub-surfaces.\n\nThe root surface in a tree of sub-surfaces is the main\nsurface. The main surface cannot be a sub-surface, because\nsub-surfaces must always have a parent.\n\nA main surface with its sub-surfaces forms a (compound) window.\nFor window management purposes, this set of wl_surface objects is\nto be considered as a single window, and it should also behave as\nsuch.\n\nThe aim of sub-surfaces is to offload some of the compositing work\nwithin a window from clients to the compositor. A prime example is\na video player with decorations and video in separate wl_surface\nobjects. This should allow the compositor to pass YUV video buffer\nprocessing to dedicated overlay hardware when possible.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_subcompositor { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_subcompositor" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_subcompositor" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "unbind from the subcompositor interface\n\nInforms the server that the client will not be using this\nprotocol object anymore. This does not affect any other\nobjects, wl_subsurface objects included.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_SUBSURFACE : u16 = 1u16 ; # [doc = "give a surface the role sub-surface\n\nCreate a sub-surface interface for the given surface, and\nassociate it with the given parent surface. This turns a\nplain wl_surface into a sub-surface.\n\nThe to-be sub-surface must not already have another role, and it\nmust not have an existing wl_subsurface object. Otherwise the\nbad_surface protocol error is raised.\n\nAdding sub-surfaces to a parent is a double-buffered operation on the\nparent (see wl_surface.commit). The effect of adding a sub-surface\nbecomes visible on the next time the state of the parent surface is\napplied.\n\nThe parent surface must not be one of the child surface's descendants,\nand the parent must be different from the child surface, otherwise the\nbad_parent protocol error is raised.\n\nThis request modifies the behaviour of wl_surface.commit request on\nthe sub-surface, see the documentation on wl_subsurface interface.\n"] pub fn get_subsurface (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , surface : waybackend :: types :: ObjectId , parent : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_SUBSURFACE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (surface)) ? ; wire_msg_builder . add_object (wayland_fd , Some (parent)) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "the to-be sub-surface is invalid"] bad_surface = 0u32 , # [doc = "the to-be sub-surface parent is invalid"] bad_parent = 1u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const bad_surface : u32 = Error :: bad_surface as u32 ; const bad_parent : u32 = Error :: bad_parent as u32 ; match value { bad_surface => Ok (Self :: bad_surface) , bad_parent => Ok (Self :: bad_parent) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "sub-surface interface to a wl_surface\n\nAn additional interface to a wl_surface object, which has been\nmade a sub-surface. A sub-surface has one parent surface. A\nsub-surface's size and position are not limited to that of the parent.\nParticularly, a sub-surface is not automatically clipped to its\nparent's area.\n\nA sub-surface becomes mapped, when a non-NULL wl_buffer is applied\nand the parent surface is mapped. The order of which one happens\nfirst is irrelevant. A sub-surface is hidden if the parent becomes\nhidden, or if a NULL wl_buffer is applied. These rules apply\nrecursively through the tree of surfaces.\n\nThe behaviour of a wl_surface.commit request on a sub-surface\ndepends on the sub-surface's mode. The possible modes are\nsynchronized and desynchronized, see methods\nwl_subsurface.set_sync and wl_subsurface.set_desync. Synchronized\nmode caches the wl_surface state to be applied when the parent's\nstate gets applied, and desynchronized mode applies the pending\nwl_surface state directly. A sub-surface is initially in the\nsynchronized mode.\n\nSub-surfaces also have another kind of state, which is managed by\nwl_subsurface requests, as opposed to wl_surface requests. This\nstate includes the sub-surface position relative to the parent\nsurface (wl_subsurface.set_position), and the stacking order of\nthe parent and its sub-surfaces (wl_subsurface.place_above and\n.place_below). This state is applied when the parent surface's\nwl_surface state is applied, regardless of the sub-surface's mode.\nAs the exception, set_sync and set_desync are effective immediately.\n\nThe main surface can be thought to be always in desynchronized mode,\nsince it does not have a parent in the sub-surfaces sense.\n\nEven if a sub-surface is in desynchronized mode, it will behave as\nin synchronized mode, if its parent surface behaves as in\nsynchronized mode. This rule is applied recursively throughout the\ntree of surfaces. This means, that one can set a sub-surface into\nsynchronized mode, and then assume that all its child and grand-child\nsub-surfaces are synchronized, too, without explicitly setting them.\n\nDestroying a sub-surface takes effect immediately. If you need to\nsynchronize the removal of a sub-surface to the parent surface update,\nunmap the sub-surface first by attaching a NULL wl_buffer, update parent,\nand then destroy the sub-surface.\n\nIf the parent wl_surface object is destroyed, the sub-surface is\nunmapped.\n\nA sub-surface never has the keyboard focus of any seat.\n\nThe wl_surface.offset request is ignored: clients must use set_position\ninstead to move the sub-surface.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wl_subsurface { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wl_subsurface" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wl_subsurface" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "remove sub-surface interface\n\nThe sub-surface interface is removed from the wl_surface object\nthat was turned into a sub-surface with a\nwl_subcompositor.get_subsurface request. The wl_surface's association\nto the parent is deleted. The wl_surface is unmapped immediately.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_POSITION : u16 = 1u16 ; # [doc = "reposition the sub-surface\n\nThis schedules a sub-surface position change.\nThe sub-surface will be moved so that its origin (top left\ncorner pixel) will be at the location x, y of the parent surface\ncoordinate system. The coordinates are not restricted to the parent\nsurface area. Negative values are allowed.\n\nThe scheduled coordinates will take effect whenever the state of the\nparent surface is applied.\n\nIf more than one set_position request is invoked by the client before\nthe commit of the parent surface, the position of a new request always\nreplaces the scheduled position from any previous request.\n\nThe initial position is 0, 0.\n"] pub fn set_position (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : i32 , y : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_POSITION) ? ; wire_msg_builder . add_i32 (wayland_fd , x) ? ; wire_msg_builder . add_i32 (wayland_fd , y) ? ; wire_msg_builder . finish () ; Ok (()) } pub const PLACE_ABOVE : u16 = 2u16 ; # [doc = "restack the sub-surface\n\nThis sub-surface is taken from the stack, and put back just\nabove the reference surface, changing the z-order of the sub-surfaces.\nThe reference surface must be one of the sibling surfaces, or the\nparent surface. Using any other surface, including this sub-surface,\nwill cause a protocol error.\n\nThe z-order is double-buffered. Requests are handled in order and\napplied immediately to a pending state. The final pending state is\ncopied to the active state the next time the state of the parent\nsurface is applied.\n\nA new sub-surface is initially added as the top-most in the stack\nof its siblings and parent.\n"] pub fn place_above (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , sibling : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , PLACE_ABOVE) ? ; wire_msg_builder . add_object (wayland_fd , Some (sibling)) ? ; wire_msg_builder . finish () ; Ok (()) } pub const PLACE_BELOW : u16 = 3u16 ; # [doc = "restack the sub-surface\n\nThe sub-surface is placed just below the reference surface.\nSee wl_subsurface.place_above.\n"] pub fn place_below (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , sibling : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , PLACE_BELOW) ? ; wire_msg_builder . add_object (wayland_fd , Some (sibling)) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_SYNC : u16 = 4u16 ; # [doc = "set sub-surface to synchronized mode\n\nChange the commit behaviour of the sub-surface to synchronized\nmode, also described as the parent dependent mode.\n\nIn synchronized mode, wl_surface.commit on a sub-surface will\naccumulate the committed state in a cache, but the state will\nnot be applied and hence will not change the compositor output.\nThe cached state is applied to the sub-surface immediately after\nthe parent surface's state is applied. This ensures atomic\nupdates of the parent and all its synchronized sub-surfaces.\nApplying the cached state will invalidate the cache, so further\nparent surface commits do not (re-)apply old state.\n\nSee wl_subsurface for the recursive effect of this mode.\n"] pub fn set_sync (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_SYNC) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_DESYNC : u16 = 5u16 ; # [doc = "set sub-surface to desynchronized mode\n\nChange the commit behaviour of the sub-surface to desynchronized\nmode, also described as independent or freely running mode.\n\nIn desynchronized mode, wl_surface.commit on a sub-surface will\napply the pending state directly, without caching, as happens\nnormally with a wl_surface. Calling wl_surface.commit on the\nparent surface has no effect on the sub-surface's wl_surface\nstate. This mode allows a sub-surface to be updated on its own.\n\nIf cached state exists when wl_surface.commit is called in\ndesynchronized mode, the pending state is added to the cached\nstate, and applied as a whole. This invalidates the cache.\n\nNote: even if a sub-surface is set to desynchronized, a parent\nsub-surface may override it to behave as synchronized. For details,\nsee wl_subsurface.\n\nIf a surface's parent surface behaves as desynchronized, then\nthe cached state is applied on set_desync.\n"] pub fn set_desync (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_DESYNC) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "wl_surface is not a sibling or the parent"] bad_surface = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const bad_surface : u32 = Error :: bad_surface as u32 ; match value { bad_surface => Ok (Self :: bad_surface) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } }
# [doc = "surface cropping and scaling\n\nThe global interface exposing surface cropping and scaling\ncapabilities is used to instantiate an interface extension for a\nwl_surface object. This extended interface will then allow\ncropping and scaling the surface contents, effectively\ndisconnecting the direct relationship between the buffer and the\nsurface size.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wp_viewporter { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wp_viewporter" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wp_viewporter" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "unbind from the cropping and scaling interface\n\nInforms the server that the client will not be using this\nprotocol object anymore. This does not affect any other objects,\nwp_viewport objects included.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_VIEWPORT : u16 = 1u16 ; # [doc = "extend surface interface for crop and scale\n\nInstantiate an interface extension for the given wl_surface to\ncrop and scale its content. If the given wl_surface already has\na wp_viewport object associated, the viewport_exists\nprotocol error is raised.\n"] pub fn get_viewport (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , surface : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_VIEWPORT) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (surface)) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "the surface already has a viewport object associated"] viewport_exists = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const viewport_exists : u32 = Error :: viewport_exists as u32 ; match value { viewport_exists => Ok (Self :: viewport_exists) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "crop and scale interface to a wl_surface\n\nAn additional interface to a wl_surface object, which allows the\nclient to specify the cropping and scaling of the surface\ncontents.\n\nThis interface works with two concepts: the source rectangle (src_x,\nsrc_y, src_width, src_height), and the destination size (dst_width,\ndst_height). The contents of the source rectangle are scaled to the\ndestination size, and content outside the source rectangle is ignored.\nThis state is double-buffered, see wl_surface.commit.\n\nThe two parts of crop and scale state are independent: the source\nrectangle, and the destination size. Initially both are unset, that\nis, no scaling is applied. The whole of the current wl_buffer is\nused as the source, and the surface size is as defined in\nwl_surface.attach.\n\nIf the destination size is set, it causes the surface size to become\ndst_width, dst_height. The source (rectangle) is scaled to exactly\nthis size. This overrides whatever the attached wl_buffer size is,\nunless the wl_buffer is NULL. If the wl_buffer is NULL, the surface\nhas no content and therefore no size. Otherwise, the size is always\nat least 1x1 in surface local coordinates.\n\nIf the source rectangle is set, it defines what area of the wl_buffer is\ntaken as the source. If the source rectangle is set and the destination\nsize is not set, then src_width and src_height must be integers, and the\nsurface size becomes the source rectangle size. This results in cropping\nwithout scaling. If src_width or src_height are not integers and\ndestination size is not set, the bad_size protocol error is raised when\nthe surface state is applied.\n\nThe coordinate transformations from buffer pixel coordinates up to\nthe surface-local coordinates happen in the following order:\n1. buffer_transform (wl_surface.set_buffer_transform)\n2. buffer_scale (wl_surface.set_buffer_scale)\n3. crop and scale (wp_viewport.set*)\nThis means, that the source rectangle coordinates of crop and scale\nare given in the coordinates after the buffer transform and scale,\ni.e. in the coordinates that would be the surface-local coordinates\nif the crop and scale was not applied.\n\nIf src_x or src_y are negative, the bad_value protocol error is raised.\nOtherwise, if the source rectangle is partially or completely outside of\nthe non-NULL wl_buffer, then the out_of_buffer protocol error is raised\nwhen the surface state is applied. A NULL wl_buffer does not raise the\nout_of_buffer error.\n\nIf the wl_surface associated with the wp_viewport is destroyed,\nall wp_viewport requests except 'destroy' raise the protocol error\nno_surface.\n\nIf the wp_viewport object is destroyed, the crop and scale\nstate is removed from the wl_surface. The change will be applied\non the next wl_surface.commit.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wp_viewport { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wp_viewport" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wp_viewport" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "remove scaling and cropping from the surface\n\nThe associated wl_surface's crop and scale state is removed.\nThe change is applied on the next wl_surface.commit.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_SOURCE : u16 = 1u16 ; # [doc = "set the source rectangle for cropping\n\nSet the source rectangle of the associated wl_surface. See\nwp_viewport for the description, and relation to the wl_buffer\nsize.\n\nIf all of x, y, width and height are -1.0, the source rectangle is\nunset instead. Any other set of values where width or height are zero\nor negative, or x or y are negative, raise the bad_value protocol\nerror.\n\nThe crop and scale state is double-buffered, see wl_surface.commit.\n"] pub fn set_source (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , x : waybackend :: types :: WlFixed , y : waybackend :: types :: WlFixed , width : waybackend :: types :: WlFixed , height : waybackend :: types :: WlFixed) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_SOURCE) ? ; wire_msg_builder . add_fixed (wayland_fd , x) ? ; wire_msg_builder . add_fixed (wayland_fd , y) ? ; wire_msg_builder . add_fixed (wayland_fd , width) ? ; wire_msg_builder . add_fixed (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_DESTINATION : u16 = 2u16 ; # [doc = "set the surface size for scaling\n\nSet the destination size of the associated wl_surface. See\nwp_viewport for the description, and relation to the wl_buffer\nsize.\n\nIf width is -1 and height is -1, the destination size is unset\ninstead. Any other pair of values for width and height that\ncontains zero or negative values raises the bad_value protocol\nerror.\n\nThe crop and scale state is double-buffered, see wl_surface.commit.\n"] pub fn set_destination (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , width : i32 , height : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_DESTINATION) ? ; wire_msg_builder . add_i32 (wayland_fd , width) ? ; wire_msg_builder . add_i32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "negative or zero values in width or height"] bad_value = 0u32 , # [doc = "destination size is not integer"] bad_size = 1u32 , # [doc = "source rectangle extends outside of the content area"] out_of_buffer = 2u32 , # [doc = "the wl_surface was destroyed"] no_surface = 3u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const bad_value : u32 = Error :: bad_value as u32 ; const bad_size : u32 = Error :: bad_size as u32 ; const out_of_buffer : u32 = Error :: out_of_buffer as u32 ; const no_surface : u32 = Error :: no_surface as u32 ; match value { bad_value => Ok (Self :: bad_value) , bad_size => Ok (Self :: bad_size) , out_of_buffer => Ok (Self :: out_of_buffer) , no_surface => Ok (Self :: no_surface) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } }
# [doc = "fractional surface scale information\n\nA global interface for requesting surfaces to use fractional scales.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wp_fractional_scale_manager_v1 { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wp_fractional_scale_manager_v1" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wp_fractional_scale_manager_v1" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "unbind the fractional surface scale interface\n\nInforms the server that the client will not be using this protocol\nobject anymore. This does not affect any other objects,\nwp_fractional_scale_v1 objects included.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_FRACTIONAL_SCALE : u16 = 1u16 ; # [doc = "extend surface interface for scale information\n\nCreate an add-on object for the the wl_surface to let the compositor\nrequest fractional scales. If the given wl_surface already has a\nwp_fractional_scale_v1 object associated, the fractional_scale_exists\nprotocol error is raised.\n"] pub fn get_fractional_scale (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , surface : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_FRACTIONAL_SCALE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (surface)) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "the surface already has a fractional_scale object associated"] fractional_scale_exists = 0u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const fractional_scale_exists : u32 = Error :: fractional_scale_exists as u32 ; match value { fractional_scale_exists => Ok (Self :: fractional_scale_exists) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } } # [doc = "fractional scale interface to a wl_surface\n\nAn additional interface to a wl_surface object which allows the compositor\nto inform the client of the preferred scale.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod wp_fractional_scale_v1 { use super :: * ; pub const MAX_VERSION : u32 = 1u32 ; pub const NAME : & str = "wp_fractional_scale_v1" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "notify of new preferred scale\n\nNotification of a new preferred scale for this surface that the\ncompositor suggests that the client should use.\n\nThe sent scale is the numerator of a fraction with a denominator of 120.\n"] fn preferred_scale (& mut self , sender_id : waybackend :: types :: ObjectId , scale : u32) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let scale = wire_msg . next_u32 () ; state . preferred_scale (wire_msg . sender_id () , scale) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("wp_fractional_scale_v1" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const DESTROY : u16 = 0u16 ; # [doc = "remove surface scale information for surface\n\nDestroy the fractional scale object. When this object is destroyed,\npreferred_scale events will no longer be sent.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } } }
# [doc = "create surfaces that are layers of the desktop\n\nClients can use this interface to assign the surface_layer role to\nwl_surfaces. Such surfaces are assigned to a \"layer\" of the output and\nrendered with a defined z-depth respective to each other. They may also be\nanchored to the edges and corners of a screen and specify input handling\nsemantics. This interface should be suitable for the implementation of\nmany desktop shell components, and a broad number of other applications\nthat interact with the desktop.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod zwlr_layer_shell_v1 { use super :: * ; pub const MAX_VERSION : u32 = 4u32 ; pub const NAME : & str = "zwlr_layer_shell_v1" ; # [doc = "Events for this interface"] pub trait EvHandler { } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("zwlr_layer_shell_v1" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const GET_LAYER_SURFACE : u16 = 0u16 ; # [doc = "create a layer_surface from a surface\n\nCreate a layer surface for an existing surface. This assigns the role of\nlayer_surface, or raises a protocol error if another role is already\nassigned.\n\nCreating a layer surface from a wl_surface which has a buffer attached\nor committed is a client error, and any attempts by a client to attach\nor manipulate a buffer prior to the first layer_surface.configure call\nmust also be treated as errors.\n\nAfter creating a layer_surface object and setting it up, the client\nmust perform an initial commit without any buffer attached.\nThe compositor will reply with a layer_surface.configure event.\nThe client must acknowledge it and is then allowed to attach a buffer\nto map the surface.\n\nYou may pass NULL for output to allow the compositor to decide which\noutput to use. Generally this will be the one that the user most\nrecently interacted with.\n\nClients can specify a namespace that defines the purpose of the layer\nsurface.\n"] pub fn get_layer_surface (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , id : waybackend :: types :: ObjectId , surface : waybackend :: types :: ObjectId , output : Option < waybackend :: types :: ObjectId > , layer : Layer , namespace : & str) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_LAYER_SURFACE) ? ; wire_msg_builder . add_new_specified_id (wayland_fd , id) ? ; wire_msg_builder . add_object (wayland_fd , Some (surface)) ? ; wire_msg_builder . add_object (wayland_fd , output) ? ; wire_msg_builder . add_u32 (wayland_fd , layer . into ()) ? ; wire_msg_builder . add_string (wayland_fd , namespace) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DESTROY : u16 = 1u16 ; # [doc = "destroy the layer_shell object\n\nThis request indicates that the client will not use the layer_shell\nobject any more. Objects that have been created through this instance\nare not affected.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "wl_surface has another role"] role = 0u32 , # [doc = "layer value is invalid"] invalid_layer = 1u32 , # [doc = "wl_surface has a buffer attached or committed"] already_constructed = 2u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const role : u32 = Error :: role as u32 ; const invalid_layer : u32 = Error :: invalid_layer as u32 ; const already_constructed : u32 = Error :: already_constructed as u32 ; match value { role => Ok (Self :: role) , invalid_layer => Ok (Self :: invalid_layer) , already_constructed => Ok (Self :: already_constructed) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } # [doc = "available layers for surfaces\n\nThese values indicate which layers a surface can be rendered in. They\nare ordered by z depth, bottom-most first. Traditional shell surfaces\nwill typically be rendered between the bottom and top layers.\nFullscreen shell surfaces are typically rendered at the top layer.\nMultiple surfaces can share a single layer, and ordering within a\nsingle layer is undefined.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Layer { # [doc = ""] background = 0u32 , # [doc = ""] bottom = 1u32 , # [doc = ""] top = 2u32 , # [doc = ""] overlay = 3u32 , } impl std :: convert :: TryFrom < u32 > for Layer { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const background : u32 = Layer :: background as u32 ; const bottom : u32 = Layer :: bottom as u32 ; const top : u32 = Layer :: top as u32 ; const overlay : u32 = Layer :: overlay as u32 ; match value { background => Ok (Self :: background) , bottom => Ok (Self :: bottom) , top => Ok (Self :: top) , overlay => Ok (Self :: overlay) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Layer" , otherwise))) , } } } impl std :: convert :: From < Layer > for u32 { fn from (value : Layer) -> u32 { value as u32 } } } # [doc = "layer metadata interface\n\nAn interface that may be implemented by a wl_surface, for surfaces that\nare designed to be rendered as a layer of a stacked desktop-like\nenvironment.\n\nLayer surface state (layer, size, anchor, exclusive zone,\nmargin, interactivity) is double-buffered, and will be applied at the\ntime wl_surface.commit of the corresponding wl_surface is called.\n\nAttaching a null buffer to a layer surface unmaps it.\n\nUnmapping a layer_surface means that the surface cannot be shown by the\ncompositor until it is explicitly mapped again. The layer_surface\nreturns to the state it had right after layer_shell.get_layer_surface.\nThe client can re-map the surface by performing a commit without any\nbuffer attached, waiting for a configure event and handling it as usual.\n"] # [allow (non_upper_case_globals , non_camel_case_types , unused_imports , unused , clippy :: too_many_arguments , clippy :: match_single_binding , clippy :: empty_docs , clippy :: just_underscores_and_digits)] pub mod zwlr_layer_surface_v1 { use super :: * ; pub const MAX_VERSION : u32 = 4u32 ; pub const NAME : & str = "zwlr_layer_surface_v1" ; # [doc = "Events for this interface"] pub trait EvHandler { # [doc = "suggest a surface change\n\nThe configure event asks the client to resize its surface.\n\nClients should arrange their surface for the new states, and then send\nan ack_configure request with the serial sent in this configure event at\nsome point before committing the new surface.\n\nThe client is free to dismiss all but the last configure event it\nreceived.\n\nThe width and height arguments specify the size of the window in\nsurface-local coordinates.\n\nThe size is a hint, in the sense that the client is free to ignore it if\nit doesn't resize, pick a smaller size (to satisfy aspect ratio or\nresize in steps of NxM pixels). If the client picks a smaller size and\nis anchored to two opposite anchors (e.g. 'top' and 'bottom'), the\nsurface will be centered on this axis.\n\nIf the width or height arguments are zero, it means the client should\ndecide its own window dimension.\n"] fn configure (& mut self , sender_id : waybackend :: types :: ObjectId , serial : u32 , width : u32 , height : u32) ; # [doc = "surface should be closed\n\nThe closed event is sent by the compositor when the surface will no\nlonger be shown. The output may have been destroyed or the user may\nhave asked for it to be removed. Further changes to the surface will be\nignored. The client should destroy the resource after receiving this\nevent, and create a new surface if they so choose.\n"] fn closed (& mut self , sender_id : waybackend :: types :: ObjectId ,) ; } pub fn event < T : EvHandler > (state : & mut T , wire_msg : & mut waybackend :: wire :: Messages < '_ > ,) -> Result < () , waybackend :: wire :: Error > { match wire_msg . op () { 0u16 => { let serial = wire_msg . next_u32 () ; let width = wire_msg . next_u32 () ; let height = wire_msg . next_u32 () ; state . configure (wire_msg . sender_id () , serial , width , height) ; Ok (()) } , 1u16 => { state . closed (wire_msg . sender_id () ,) ; Ok (()) } , otherwise => Err (waybackend :: wire :: Error :: UnrecognizedEventOpCode (("zwlr_layer_surface_v1" , otherwise))) } } # [doc = "Requests for this interface"] pub mod req { use super :: * ; pub const SET_SIZE : u16 = 0u16 ; # [doc = "sets the size of the surface\n\nSets the size of the surface in surface-local coordinates. The\ncompositor will display the surface centered with respect to its\nanchors.\n\nIf you pass 0 for either value, the compositor will assign it and\ninform you of the assignment in the configure event. You must set your\nanchor to opposite edges in the dimensions you omit; not doing so is a\nprotocol error. Both values are 0 by default.\n\nSize is double-buffered, see wl_surface.commit.\n"] pub fn set_size (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , width : u32 , height : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_SIZE) ? ; wire_msg_builder . add_u32 (wayland_fd , width) ? ; wire_msg_builder . add_u32 (wayland_fd , height) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_ANCHOR : u16 = 1u16 ; # [doc = "configures the anchor point of the surface\n\nRequests that the compositor anchor the surface to the specified edges\nand corners. If two orthogonal edges are specified (e.g. 'top' and\n'left'), then the anchor point will be the intersection of the edges\n(e.g. the top left corner of the output); otherwise the anchor point\nwill be centered on that edge, or in the center if none is specified.\n\nAnchor is double-buffered, see wl_surface.commit.\n"] pub fn set_anchor (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , anchor : Anchor) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_ANCHOR) ? ; wire_msg_builder . add_u32 (wayland_fd , anchor . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_EXCLUSIVE_ZONE : u16 = 2u16 ; # [doc = "configures the exclusive geometry of this surface\n\nRequests that the compositor avoids occluding an area with other\nsurfaces. The compositor's use of this information is\nimplementation-dependent - do not assume that this region will not\nactually be occluded.\n\nA positive value is only meaningful if the surface is anchored to one\nedge or an edge and both perpendicular edges. If the surface is not\nanchored, anchored to only two perpendicular edges (a corner), anchored\nto only two parallel edges or anchored to all edges, a positive value\nwill be treated the same as zero.\n\nA positive zone is the distance from the edge in surface-local\ncoordinates to consider exclusive.\n\nSurfaces that do not wish to have an exclusive zone may instead specify\nhow they should interact with surfaces that do. If set to zero, the\nsurface indicates that it would like to be moved to avoid occluding\nsurfaces with a positive exclusive zone. If set to -1, the surface\nindicates that it would not like to be moved to accommodate for other\nsurfaces, and the compositor should extend it all the way to the edges\nit is anchored to.\n\nFor example, a panel might set its exclusive zone to 10, so that\nmaximized shell surfaces are not shown on top of it. A notification\nmight set its exclusive zone to 0, so that it is moved to avoid\noccluding the panel, but shell surfaces are shown underneath it. A\nwallpaper or lock screen might set their exclusive zone to -1, so that\nthey stretch below or over the panel.\n\nThe default value is 0.\n\nExclusive zone is double-buffered, see wl_surface.commit.\n"] pub fn set_exclusive_zone (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , zone : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_EXCLUSIVE_ZONE) ? ; wire_msg_builder . add_i32 (wayland_fd , zone) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_MARGIN : u16 = 3u16 ; # [doc = "sets a margin from the anchor point\n\nRequests that the surface be placed some distance away from the anchor\npoint on the output, in surface-local coordinates. Setting this value\nfor edges you are not anchored to has no effect.\n\nThe exclusive zone includes the margin.\n\nMargin is double-buffered, see wl_surface.commit.\n"] pub fn set_margin (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , top : i32 , right : i32 , bottom : i32 , left : i32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_MARGIN) ? ; wire_msg_builder . add_i32 (wayland_fd , top) ? ; wire_msg_builder . add_i32 (wayland_fd , right) ? ; wire_msg_builder . add_i32 (wayland_fd , bottom) ? ; wire_msg_builder . add_i32 (wayland_fd , left) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_KEYBOARD_INTERACTIVITY : u16 = 4u16 ; # [doc = "requests keyboard events\n\nSet how keyboard events are delivered to this surface. By default,\nlayer shell surfaces do not receive keyboard events; this request can\nbe used to change this.\n\nThis setting is inherited by child surfaces set by the get_popup\nrequest.\n\nLayer surfaces receive pointer, touch, and tablet events normally. If\nyou do not want to receive them, set the input region on your surface\nto an empty region.\n\nKeyboard interactivity is double-buffered, see wl_surface.commit.\n"] pub fn set_keyboard_interactivity (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , keyboard_interactivity : KeyboardInteractivity) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_KEYBOARD_INTERACTIVITY) ? ; wire_msg_builder . add_u32 (wayland_fd , keyboard_interactivity . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } pub const GET_POPUP : u16 = 5u16 ; # [doc = "assign this layer_surface as an xdg_popup parent\n\nThis assigns an xdg_popup's parent to this layer_surface.  This popup\nshould have been created via xdg_surface::get_popup with the parent set\nto NULL, and this request must be invoked before committing the popup's\ninitial state.\n\nSee the documentation of xdg_popup for more details about what an\nxdg_popup is and how it is used.\n"] pub fn get_popup (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , popup : waybackend :: types :: ObjectId) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , GET_POPUP) ? ; wire_msg_builder . add_object (wayland_fd , Some (popup)) ? ; wire_msg_builder . finish () ; Ok (()) } pub const ACK_CONFIGURE : u16 = 6u16 ; # [doc = "ack a configure event\n\nWhen a configure event is received, if a client commits the\nsurface in response to the configure event, then the client\nmust make an ack_configure request sometime before the commit\nrequest, passing along the serial of the configure event.\n\nIf the client receives multiple configure events before it\ncan respond to one, it only has to ack the last configure event.\n\nA client is not required to commit immediately after sending\nan ack_configure request - it may even ack_configure several times\nbefore its next surface commit.\n\nA client may send multiple ack_configure requests before committing, but\nonly the last request sent before a commit indicates which configure\nevent the client really is responding to.\n"] pub fn ack_configure (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , serial : u32) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , ACK_CONFIGURE) ? ; wire_msg_builder . add_u32 (wayland_fd , serial) ? ; wire_msg_builder . finish () ; Ok (()) } pub const DESTROY : u16 = 7u16 ; # [doc = "destroy the layer_surface\n\nThis request destroys the layer surface.\n\nTHIS IS A DESTRUCTOR"] pub fn destroy (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId ,) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , DESTROY) ? ; wire_msg_builder . finish () ; Ok (()) } pub const SET_LAYER : u16 = 8u16 ; # [doc = "change the layer of the surface\n\nChange the layer that the surface is rendered on.\n\nLayer is double-buffered, see wl_surface.commit.\n"] pub fn set_layer (backend : & mut waybackend :: Waybackend , sender_id : waybackend :: types :: ObjectId , layer : zwlr_layer_shell_v1 :: Layer) -> Result < () , waybackend :: wire :: Error > { let waybackend :: Waybackend { wire_msg_builder , wayland_fd , .. } = backend ; wire_msg_builder . add_header (wayland_fd , sender_id , SET_LAYER) ? ; wire_msg_builder . add_u32 (wayland_fd , layer . into ()) ? ; wire_msg_builder . finish () ; Ok (()) } } # [doc = "types of keyboard interaction possible for a layer shell surface\n\nTypes of keyboard interaction possible for layer shell surfaces. The\nrationale for this is twofold: (1) some applications are not interested\nin keyboard events and not allowing them to be focused can improve the\ndesktop experience; (2) some applications will want to take exclusive\nkeyboard focus.\n"] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum KeyboardInteractivity { # [doc = "no keyboard focus is possible\n\nThis value indicates that this surface is not interested in keyboard\nevents and the compositor should never assign it the keyboard focus.\n\nThis is the default value, set for newly created layer shell surfaces.\n\nThis is useful for e.g. desktop widgets that display information or\nonly have interaction with non-keyboard input devices.\n"] None = 0u32 , # [doc = "request exclusive keyboard focus\n\nRequest exclusive keyboard focus if this surface is above the shell surface layer.\n\nFor the top and overlay layers, the seat will always give\nexclusive keyboard focus to the top-most layer which has keyboard\ninteractivity set to exclusive. If this layer contains multiple\nsurfaces with keyboard interactivity set to exclusive, the compositor\ndetermines the one receiving keyboard events in an implementation-\ndefined manner. In this case, no guarantee is made when this surface\nwill receive keyboard focus (if ever).\n\nFor the bottom and background layers, the compositor is allowed to use\nnormal focus semantics.\n\nThis setting is mainly intended for applications that need to ensure\nthey receive all keyboard events, such as a lock screen or a password\nprompt.\n"] Exclusive = 1u32 , # [doc = "request regular keyboard focus semantics\n\nThis requests the compositor to allow this surface to be focused and\nunfocused by the user in an implementation-defined manner. The user\nshould be able to unfocus this surface even regardless of the layer\nit is on.\n\nTypically, the compositor will want to use its normal mechanism to\nmanage keyboard focus between layer shell surfaces with this setting\nand regular toplevels on the desktop layer (e.g. click to focus).\nNevertheless, it is possible for a compositor to require a special\ninteraction to focus or unfocus layer shell surfaces (e.g. requiring\na click even if focus follows the mouse normally, or providing a\nkeybinding to switch focus between layers).\n\nThis setting is mainly intended for desktop shell components (e.g.\npanels) that allow keyboard interaction. Using this option can allow\nimplementing a desktop shell that can be fully usable without the\nmouse.\n"] OnDemand = 2u32 , } impl std :: convert :: TryFrom < u32 > for KeyboardInteractivity { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const None : u32 = KeyboardInteractivity :: None as u32 ; const Exclusive : u32 = KeyboardInteractivity :: Exclusive as u32 ; const OnDemand : u32 = KeyboardInteractivity :: OnDemand as u32 ; match value { None => Ok (Self :: None) , Exclusive => Ok (Self :: Exclusive) , OnDemand => Ok (Self :: OnDemand) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("KeyboardInteractivity" , otherwise))) , } } } impl std :: convert :: From < KeyboardInteractivity > for u32 { fn from (value : KeyboardInteractivity) -> u32 { value as u32 } } # [doc = ""] # [repr (u32)] # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] pub enum Error { # [doc = "provided surface state is invalid"] invalid_surface_state = 0u32 , # [doc = "size is invalid"] invalid_size = 1u32 , # [doc = "anchor bitfield is invalid"] invalid_anchor = 2u32 , # [doc = "keyboard interactivity is invalid"] invalid_keyboard_interactivity = 3u32 , } impl std :: convert :: TryFrom < u32 > for Error { type Error = waybackend :: wire :: Error ; # [allow (non_upper_case_globals)] fn try_from (value : u32) -> Result < Self , Self :: Error > { const invalid_surface_state : u32 = Error :: invalid_surface_state as u32 ; const invalid_size : u32 = Error :: invalid_size as u32 ; const invalid_anchor : u32 = Error :: invalid_anchor as u32 ; const invalid_keyboard_interactivity : u32 = Error :: invalid_keyboard_interactivity as u32 ; match value { invalid_surface_state => Ok (Self :: invalid_surface_state) , invalid_size => Ok (Self :: invalid_size) , invalid_anchor => Ok (Self :: invalid_anchor) , invalid_keyboard_interactivity => Ok (Self :: invalid_keyboard_interactivity) , otherwise => Err (waybackend :: wire :: Error :: InvalidEnumDiscriminant (("Error" , otherwise))) , } } } impl std :: convert :: From < Error > for u32 { fn from (value : Error) -> u32 { value as u32 } } waybackend :: bitflags :: bitflags ! { # [derive (Debug , Clone , Copy , PartialEq , Eq , Hash)] # [doc = ""] pub struct Anchor : u32 { # [doc = "the top edge of the anchor rectangle"] const TOP = 1u32 ; # [doc = "the bottom edge of the anchor rectangle"] const BOTTOM = 2u32 ; # [doc = "the left edge of the anchor rectangle"] const LEFT = 4u32 ; # [doc = "the right edge of the anchor rectangle"] const RIGHT = 8u32 ; const _ = ! 0 ; } } impl std :: convert :: TryFrom < u32 > for Anchor { type Error = waybackend :: wire :: Error ; fn try_from (value : u32) -> Result < Self , Self :: Error > { Ok (Self :: from_bits_retain (value)) } } impl std :: convert :: From < Anchor > for u32 { fn from (value : Anchor) -> u32 { value . bits () } } }
